;********************************************************************
;AV5S.ASM
;Kernel Operators for hardware spezific  I/O
;********************************************************************

AREF:  ;  ( Vref -- )  **********************************************
;********************************************************************
//to improve SLEEP and other low power modes,
// switch ADC OFF as an easy to handle extra feature 
	clr r25
#if (CPU==32)
	out ADCSR,r25      
#else
	sts ADCSRA,r25
#endif
	andi AVIFLAGS,0x3F		;clear VREF0, VFREF1 bits
	tst TOSL				;external VREF
	breq ARF80
	cpi TOSL,1
	brne ARF10
	ori AVIFLAGS,0x40		;set REFS0->VREF=internal Vcc
	rjmp ARF80
ARF10:
	cpi TOSL,2
	breq ARF20
	jmp parameterErr
ARF20:
	ori AVIFLAGS,0xC0		;set REFS0,REFS1->VREF=internal 1.1V
ARF80:
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


RADC:  ;  ( b -- x )  ***********************************************
;use ADC with VCC internally fed to VREF. VREF pin should filtered with C
;switch ADC OFF only by use of IP, IZ, OH. OL of the corresponding pin
;the port bit is converted into HiZ before the ADC gets initialized
;********************************************************************
	mov TEMP2,AVIFLAGS
	andi TEMP2,0xC0		;seprate VREF0,VREF1 = VREF selection

	cpi TOSL,ANAIN
	brcs RAD10
	jmp parameterErr
RAD10:
#if (CPU==3244)
	cpi TOSL,2
	brcs RAD20
	inc TOSL			;transform user input 2...5 into HW address 4...7
	inc TOSL
RAD20:
#endif
;init port HiZ
	ldi ZH,HIGH(BITMASK*2)
	ldi ZL,LOW(BITMASK*2)
	add ZL,TOSL
	lpm r24,Z
	clr ZH
#if (CPU==32)
	ldi ZL,0x3A				;DDRA+0x20
#endif
#if ((CPU==644) || (CPU==1284))
	ldi ZL,0x21				;DDRA+0x20
	lds TEMP1,ADCSRA
#endif
#if ((CPU==168) || (CPU==328))
	ldi ZL,0x27				;DDRC+0x20
#endif
#if (CPU==3244)
	ldi ZL,0x30				;DDRF+0x20
#endif
	com r24					;one's complement
	ldd r25,Z+1
	and r25,r24             ;clear Port-Bit
	std Z+1,r25
	ld r25,Z
	and r25,r24           	;clear DDR-Bit = input HiZ
	st Z,r25

	or TOSL,TEMP2			;set REFS0,REFS1 bits as configured by AREF
#if (CPU==32)
	in TEMP2, ADMUX
	out ADMUX,TOSL	
#else
	lds TEMP2,ADMUX
	sts ADMUX,TOSL
#endif

#ifdef SPEED_18M
	ldi r25,(1<<ADEN|1<<ADSC|1<<ADIF|7) ;0xD7
		;ADCEnable, Single conversion,divide128=143kHz
#endif
#ifdef SPEED_16M
	ldi r25,(1<<ADEN|1<<ADSC|7) ;0xD7
		;ADCEnable, Single conversion,divide128=125kHz
#endif
#ifdef SPEED_14M
	ldi r25,(1<<ADEN|1<<ADSC|7) ;0xD7
		;ADCEnable, Single conversion,divide128=109kHz
#endif
#ifdef SPEED_11M
	ldi r25,(1<<ADEN|1<<ADSC|7) ;0xD7
		;ADCEnable, Single conversion,divide128=86kHz
#endif
#ifdef SPEED_7M
	ldi r25,(1<<ADEN|1<<ADSC|6) ;0xD6
		;ADCEnable, Single conversion,divide64=115kHz
#endif
#ifdef SPEED_1M8
	ldi r25,(1<<ADEN|1<<ADSC|4) ;0xD4
		;ADCEnable, Single conversion,divid16=115kHz
#endif

#if (CPU==32)
	in TEMP1,ADCSR
	out ADCSR,r25
#else 
	lds TEMP1,ADCSRA
	sts ADCSRA,r25
#endif
	cp TOSL,TEMP2
	breq RAD40				;jump if ADC setup has not changed
	andi TEMP1,0x80			;ADEN bit set?
	brne RAD40
//if ADC new config,perform an extra ADC readout to improve first ADC value
RAD30:
#if (CPU==32)
	sbic ADCSR,ADSC         ;skip, if conversion ready
#else
	lds TEMP2,ADCSRA
	sbrc TEMP2,ADSC         ;skip, if conversion ready
#endif
	rjmp RAD30

RAD40:
#if (CPU==32)
	out ADCSR,r25
#else
	sts ADCSRA,r25
#endif

RAD50:
#if (CPU==32)
	sbic ADCSR,ADSC         ;skip, if conversion ready
#else
	lds r25,ADCSRA
	sbrc r25,ADSC          	;skip, if conversion ready
#endif
	rjmp RAD50

#if (CPU==32)
	in TOSL,ADCL
	in TOSH,ADCH
#else
	lds TOSL,ADCL
	lds TOSH,ADCH
#endif
	andi TOSH,3				;limit result to 10bit
	ret


tosToMask:   ; (TOS -- TOS) *****************************************
; Return r24 as bit mask
; Return Z as pointer to corresponding I/O register
; Return TOSH as simplified port number
; ATmega32 has different register adresses
; PortA only at models with 40pin DIL package (ATmega32,644,1284)
; Ports E6 and F provided only at ATmega32U4 
;********************************************************************
	mov TOSH,TOSL
	swap TOSH
	andi TOSH,0x0F          ;now Port-letter in hex A,B,C,D, (E,F mega32U4 only)
	andi TOSL,0x0F          ;Port Pin No
	cpi TOSL,8
	brcc TOM90
TOM10:
	ldi ZH,HIGH(BITMASK*2)
	ldi ZL,LOW(BITMASK*2)
	add ZL,TOSL
	lpm r24,Z				;pin number ->byte mask

#if ((CPU==168) || (CPU==328))
	cpi TOSH,0x0B			 ;range test
	brcs TOM90
	cpi TOSH,0x0E
	brcc TOM90
	subi TOSH,0x0B			;range 0,1,2
	ldi ZL,0x24           	;addr(DDRB)=4 + 0x20 + 3*TOSH
							;see ATmega88/168 manual p.22
	add ZL,TOSH				;addr(DDRC)=7
	add ZL,TOSH				;addr(DDRD)=0x0A
	add ZL,TOSH
	inc TOSH				;now range 1,2,3

	cpi TOSH,3
	brne TOM30
	andi r24,0xBC			;PD0,PD1 reserved for USART0, PD6 reservedfor LED		
	breq TOM90				;so mask of application available pins is 0xBC
TOM30:
#endif
#if ((CPU==644) || (CPU==1284))
	cpi TOSH,0x0A            ;range test
	brcs TOM90  
	cpi TOSH,0x0E
	brcc TOM90
	subi TOSH,0x0A			;range 0,1,2,3
	ldi ZL,0x21             ;addr(DDRA)=1 + 0x20 + 3*TOSH
							;see ATmega644/1284 manual p.23
	add ZL,TOSH				;addr(DDRB)=4
	add ZL,TOSH				;addr(DDRC)=7
	add ZL,TOSH				;addr(DDRD)=0x0A

	cpi TOSH,1
	brne TOM30
	andi r24,0xFE			;PB0 reserved for LED			;
	breq TOM90				;so mask of application available pins is 0xFE
TOM30:
	cpi TOSH,3
	brne TOM40
	sbrs XFLAGS,USART1_ON
	andi r24,0xFC			;PD3 - PD7  available for user application			;
	sbrc XFLAGS,USART1_ON
	andi r24,0xF0			;PD0-PD3 reserved for USARTs
	breq TOM90				;so mask of application available pins is 0xF0
TOM40:
#endif
#if (CPU==3244)
	cpi TOSH,0x0B			 ;range test
	brcs TOM90
	cpi TOSH,0x10
	brcc TOM90
	subi TOSH,0x0B			;range 0,1,2
	ldi ZL,0x24             ;addr(DDRB)=4 + 0x20 + 3*TOSH
							;see ATmega32U4 manual p.26
	add ZL,TOSH				;addr(DDRC)=7
	add ZL,TOSH				;addr(DDRD)=0x0A
	add ZL,TOSH				;addr(DDRF)=0x10
	inc TOSH				;now range 1,2,3,4,5

	cpi TOSH,1				;PortB fully implemented, speed up
	breq TOM50				;LEONARDO B0 user accessible LED, OL=ON/else=OFF
	cpi TOSH,5
	brne TOM20
	andi r24,0xF3			;PF2,3 not implemented, mask of available=0xF3
	breq TOM90				;if bit not within this range,then PF2 or PF3
	rjmp TOM50				;PB7 must be handled with care,is ENC1 ref.pin
TOM20:
	cpi TOSH,3
	brne TOM30
#ifdef LEONARDO
	sbrs XFLAGS,USART1_ON
	andi r24,0xDF			;PD0 - PD4,PD6,PD7, available for user application
	sbrc XFLAGS,USART1_ON
	andi r24,0xD3			;PD0,PD1,PD4,PD56,PD7 available for application
	breq TOM90				;PD2/PD3 reserved for USART1, PD5 reserved for LED
							;PD0,PD1,PD4 must be handled with care, 
							;because these are used by CNT0,ENC0,CNT1 !!
#else
	sbrs XFLAGS,USART1_ON
	andi r24,0xBF			;PD0 - PD4,PD7  available for user application
	sbrc XFLAGS,USART1_ON
	andi r24,0xB3			;only PD0,PD1,PD4,PD5,PD7  available for application
	breq TOM90				;PD2/PD3 reserved for USART1, PD6 reserved for LED
							;PD0,PD1,PD4 must be handled with care, 
							;because these are used by CNT0,ENC0,CNT1 !!
							;PD5 is user accessible yellow LED on Arduino PCB
#endif
	rjmp TOM50					
TOM30:
	cpi TOSH,2
	brne TOM40
	andi r24,0xC0			;only PC6, PC7 implemented, mask of available=0xC0
	breq TOM90				;LEONARDO PC7: user accessable yellow LED on PCB
	rjmp TOM50		
TOM40:
	cpi TOSH,4
	brne TOM50
	andi r24,0x40			;only PE6  implemented, mask of available=0x40
	breq TOM90			
TOM50:
#endif	;(CPU==3244)
#if (CPU==32)
	cpi TOSH,0x0A			;range test
	brcs TOM90              
	cpi TOSH,0x0E
	brcc TOM90
	subi TOSH,0x0A			;range 0,1,2,3
	ldi ZL,0x3A            	;addr(DDRA)=0x1A + 0x20 - 3*TOSH
							;see ATmega32 manual p.17
	sub ZL,TOSH				;addr(DDRB)=0x17
	sub ZL,TOSH				;addr(DDRC)=0x14
	sub ZL,TOSH				;addr(DDRD)=0x11
	cpi TOSH,1
	brne TOM30
	andi r24,0xFE			;PB0 for LED
	breq TOM90				;so mask of application available pins is 0xFE
TOM30:
	cpi TOSH,3
	brne TOM40
	andi r24,0xFC			;PD0,PD1 reserved for USART
	breq TOM90				;so mask of application available pins is 0xFC
TOM40:
#endif
	clr ZH
	ret

TOM90:  jmp parameterErr


PBOH:  ;  ( b  -- )  *************************************************
; configure PortBit as output and set HIGH
; TOS format:8bit,HighNibble=0xA,B,C or D,(E;F) Low nibble=pinNum 0...7
; ATmega32 has different register adresses
; PortA provided only at models with 40pin DIL package (ATmega32,644,1284)
; Ports E6 and F provided only at ATmega32U4 
;********************************************************************
	rcall handleBusy		;complex return values see there
	rcall tosToMask
	ldd TOSL,Z+1
	or TOSL,r24             ;set Port-Bit
	std Z+1,TOSL
	ld TOSL,Z
	or TOSL,r24             ;setDDR-Bit
	st Z,TOSL
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


PBOL:  ;  ( b -- )  *************************************************
; configure PortBit as output and set LOW
; TOS format:8bit,HighNibble=0xA,B,C or D,(E;F) Low nibble=pinNum 0...7
; ATmega32 has different register adresses
; PortA only provided at models with 40pin DIL package (ATmega32,644,1284)
; Ports E6 and F provided only at ATmega32U4
;********************************************************************
	rcall handleBusy		;complex return values see there
	rcall tosToMask			;complex return values see there
	ld TOSL,Z
	or TOSL,r24             ;set DDR-Bit
	st Z,TOSL
	ldd TOSL,Z+1
	com r24					;one's complement
	and TOSL,r24
	std Z+1,TOSL            ;clear Port-Bit
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


PORTH:  ;  ( b -- )  *************************************************
; set PortBit Hard High,short version implies pin is already config.as output
; TOS format:8bit,HighNibble=0xA,B,C or D,(E;F) Low nibble=pinNum 0...7
; ATmega32 has different register adresses
; PortA only provided at models with 40pin DIL package (ATmega32,644,1284)
; Ports E6 and F provided only at ATmega32U4
;********************************************************************
	rcall tosToMask
	ldd TOSL,Z+1
	or TOSL,r24             ;set Port-Bit
	std Z+1,TOSL
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


PORTL: ;  ( b -- )  **************************************************
; set PortBit Low,short version implies pin is already config.as output
; TOS format:8bit,HighNibble=0xA,B,C or D,(E;F) Low nibble=pinNum 0...7
; ATmega32 has different register adresses
; PortA only provided at models with 40pin DIL package (ATmega32,644,1284)
; Ports E6 and F provided only at ATmega32U4
;********************************************************************
	rcall tosToMask			;complex return values see there
	ldd TOSL,Z+1
	com r24					;one's complement
	and TOSL,r24
	std Z+1,TOSL            ;Port-Bit lschen
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


PBIH:  ;  ( b -- )  *************************************************
; set PortBit Soft High
; TOS format:8bit,HighNibble=0xA,B,C or D,(E;F) Low nibble=pinNum 0...7
; ATmega32 has different register adresses
; PortA only provided at models with 40pin DIL package (ATmega32,644,1284)
; Ports E6 and F provided only at ATmega32U4
;********************************************************************
	rcall handleBusy
	rcall tosToMask			;complex return values see there
	ldd TOSL,Z+1
	or TOSL,r24             ;setPort-Bit
	std Z+1,TOSL
	com r24					;one's complement
	ld TOSL,Z
	and TOSL,r24           	;clear DDR-Bit
	st Z,TOSL
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


PBIZ:  ;  ( b -- )  *************************************************
; set PortBit High Z
; TOS format:8bit,HighNibble=0xA,B,C or D,(E;F) Low nibble=pinNum 0...7
; ATmega32 has different register adresses
; PortA only provided at models with 40pin DIL package (ATmega32,644,1284)
; Ports E6 and F provided only at ATmega32U4
;********************************************************************
	rcall handleBusy		;complex return values see there
	rcall tosToMask
	com r24					;one's complement
	ld TOSL,Z
	and TOSL,r24            ;clear DDR-Bitn
	st Z,TOSL
	ldd TOSL,Z+1
	and TOSL,r24            ;clear Port-Bit
	std Z+1,TOSL
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


handleBusy:  ; ( -- ) ***********************************************
;if port with OC1A or OC2A/B is addressed, switch off Freq-Generation
;if SPI ports are addressed, switch off SPI
;********************************************************************
#if ((CPU==168) || (CPU==328))
	cpi TOSL,0xB1
	brne HBY10
	sbrs XFLAGS,TIM1BUSY
	rjmp HBY10
;switch OFF Timer1
	clr r25
	sts TCCR1B,r25
	sts TCCR1A,r25
	cbr XFLAGS,m_TIM1BUSY
HBY10:  
	cpi TOSL,0xD3
	brne HBY20
	sbrs XFLAGS,PWMBUSY
	rjmp HBY20
;switch OFF Timer2
	clr r25
	sts TCCR2A,r25
	sts TCCR2B,r25
	cbr XFLAGS,m_PWMBUSY
HBY20:
	sbrs XFLAGS,SPIBUSY
	ret
	cpi TOSL,0xB3			;SPI is PB3,PB4,PB5
	breq HBY30
	cpi TOSL,0xB4
	breq HBY30
	cpi TOSL,0xB5
	brne HBY80
HBY30:
	clr r25
	out SPCR,r25
	cbr XFLAGS,m_SPIBUSY
HBY80:
	ret
#endif
#if ((CPU==644) || (CPU==1284))
	cpi TOSL,0xD5
	brne HBY10
	sbrs XFLAGS,TIM1BUSY
	rjmp HBY10
;switch OFF Timer1
	clr r25
	sts TCCR1A,r25
	sts TCCR1B,r25
	cbr XFLAGS,m_TIM1BUSY
HBY10:  
	cpi TOSL,0xD7
	brne HBY20
	sbrs XFLAGS,PWMBUSY
	rjmp HBY20
;switch OFF Timer2
	clr r25
	sts TCCR2A,r25
	sts TCCR2B,r25
	cbr XFLAGS,m_PWMBUSY
HBY20:
	sbrs XFLAGS,SPIBUSY
	ret
	cpi TOSL,0xB5			;SPI is PB5,PB6,PB7
	breq HBY30
	cpi TOSL,0xB6
	breq HBY30
	cpi TOSL,0xB7
	brne HBY80
HBY30:
	clr r25
	out SPCR,r25
	cbr XFLAGS,m_SPIBUSY
HBY80:
	ret
#endif
#if (CPU==3244)
	cpi TOSL,0xB6
	brne HBY10
	sbrs XFLAGS,TIM1BUSY
	rjmp HBY10
;switch OFF Timer1
	clr r25
	sts TCCR1A,r25
	sts TCCR1B,r25
	cbr XFLAGS,m_TIM1BUSY
HBY10:  
	cpi TOSL,0xC6
	brne HBY20
	sbrs XFLAGS,PWMBUSY
	rjmp HBY20
;switch OFF Timer3
	clr r25
	sts TCCR3A,r25
	sts TCCR3B,r25
	cbr XFLAGS,m_PWMBUSY
HBY20:
	sbrs XFLAGS,SPIBUSY
	ret
	cpi TOSL,0xB1			;SPI is PB1,PB2,PB3
	breq HBY30
	cpi TOSL,0xB2
	breq HBY30
	cpi TOSL,0xB3
	brne HBY80
HBY30:
	clr r25
	out SPCR,r25
	cbr XFLAGS,m_SPIBUSY
HBY80:
	ret
#endif
#if (CPU==32)
	cpi TOSL,0xD5
	brne HBY10
	sbrs XFLAGS,TIM1BUSY
	rjmp HBY10
;switch OFF Timer1
	clr r25
	out TCCR1A,r25
	out TCCR1B,r25
	cbr XFLAGS,m_TIM1BUSY
HBY10:  
	cpi TOSL,0xD7
	brne HBY20
	sbrs XFLAGS,PWMBUSY
	rjmp HBY20
;switch OFF Timer2
	clr r25
	out TCCR2,r25
	cbr XFLAGS,m_PWMBUSY

HBY20:
	sbrs XFLAGS,SPIBUSY
	ret
	cpi TOSL,0xB5			;SPI is PB5,PB6,PB7
	breq HBY30
	cpi TOSL,0xB6
	breq HBY30
	cpi TOSL,0xB7
	brne HBY80
HBY30:
	clr r25
	out SPCR,r25
	cbr XFLAGS,m_SPIBUSY
HBY80:
	ret
#endif


RDIGIN:  ;  ( b -- TRUE|FALSE)  *************************************
; test PortBit
; ATmega32 has different register adresses
; PortA only at models with 40pin DIL package (ATmega32,644,1284)
;********************************************************************
	rcall tosToMask
	dec ZL                  ;PIN addressieren
	ld TOSL,Z
	and TOSL,r24            ;separate adressed bit
	breq PRD10              ;implies FALSE flag
	ldi TOSL,1              ;normalize TRUE flag
PRD10:  
	clr TOSH
	ret



CNT0:  ;  ( -- x )  ***********************************************
;counts negative edges at INT0-input
;16 bit format, cyclic overflow
;Low-Level process is done in "countInt0"
;********************************************************************
	cli
	st -X,TOSH			;;DUP = release TOS
	st -X,TOSL
	lds TOSH,CNT0H
	lds TOSL,CNT0L
	nop
	sei
	ret



CNT0Z:  ;  ( -- )  ************************************************
; reset CNT0-Count- and Encode0-counter
;********************************************************************
	cli
	clr r25
	sts CNT0H,r25
	sts CNT0L,r25
	sts ENC0H,r25
	sts ENC0L,r25
	nop
	sei
	ret


ENCODE0:  ;  ( -- x )  **********************************************
;counts negative edges at INT0-input
;counter direction depends on ENCODE0 pin level:counts up,if HIGH, else down
;16 bit format, cyclic overflow
;Low-Level process is done in "countInt0"
;********************************************************************
	cli
	st -X,TOSH			;;DUP = release TOS
	st -X,TOSL
	lds TOSH,ENC0H
	lds TOSL,ENC0L
	nop
	sei
	ret


#if ((CPU==32) || (CPU==3244) || (CPU==644) || (CPU==1284))
CNTXX:  ;  ( -- x )  ************************************************
;counts negative edges at INT1 or INT2-input
;16 bit format, cyclic overflow
;Low-Level process is done in "countIntX"
;********************************************************************
	cli
	st -X,TOSH			;;DUP = release TOS
	st -X,TOSL
	lds TOSH,CNTXH
	lds TOSL,CNTXL
	nop
	sei
	ret


CNTXZ:  ;  ( -- )  ************************************************
; reset CNTX-Count- and EncodeX-counter
;********************************************************************
	cli
	clr r25
	sts CNTXH,r25
	sts CNTXL,r25
	sts ENCXH,r25
	sts ENCXL,r25
	nop
	sei
	ret


ENCODEX:  ;  ( -- x )  **********************************************
;counts negative edges at INT1 or INT2-input
;counter direction depends on ENCODEX pin level:counts up,if HIGH, else down
;16 bit format, cyclic overflow
;Low-Level process is done in "countIntX"
;********************************************************************
	cli
	st -X,TOSH			;;DUP = release TOS
	st -X,TOSL
	lds TOSH,ENCXH
	lds TOSL,ENCXL
	nop
	sei
	ret
#endif


WAVE:  ;  ( x -- )  *************************************************
; OC1A resets port OC1A to Low and resets Timer1. No interrupt
; OC1A is pin PB1(ATmega168,328), PD5(40pinDIL), PB5(ATmega32U4)
; WAVEHI must be initialized before call of WAVE
; invalid input is trnasformed into simple default values
; WAVELO-parameter held in registers r14,r15
; WAVEHI.. in SRAM
; switch off only with mit IP,IZ,OH,OL
;********************************************************************
;calculate new periode. WAVEHI preset must be set, else simple default
WAV05:  
	cli
	tst TOSH
	brne WAV10
	cpi TOSL,0x10
	brcc WAV10
	ldi TOSL,0x10			;MIN!!,else interrupt generates permanent HIGH out
WAV10:
	mov ZL,TOSL
	mov ZH,TOSH             ;keep original in mind
	lds TEMP1,WAVEHIL
	lds TEMP2,WAVEHIH
	sub TOSL,TEMP1
	sbc TOSH,TEMP2
	brcs WAV20              ;if no Carry, then WAVEHI>period
	brne WAV50              ;then set default-WAVEHI 
WAV20:
	clr r25
	sts WAVEHIH,r25
	ldi TEMP1,8
	sts WAVEHIL,TEMP1       ;default preset:Low-Phase=1
	sbiw ZL,8               ;Z was total period, subtract Low-Phase
	mov WAVELOL,ZL
	mov WAVELOH,ZH
	rjmp WAV60

WAV50:  
	mov WAVELOL,TOSL
	mov WAVELOH,TOSH
WAV60:  
;check if timer aready in WAVE-operation
#if (CPU==32)
	in TEMP1,TCCR1A        ;Bit positions TCCR1A like other CPU's
#else
	lds TEMP1,TCCR1A
#endif
	andi TEMP1,0xBF         ;don't care Bit6=COM1A0, is changing
	cpi TEMP1,0x80			;only COM1A1 set = WAVE active, PWGM12->CTC mode
	brne WAV70             
#if (CPU==32)
	in TEMP1,TCCR1B
#else
	lds TEMP1,TCCR1B
#endif

#ifdef SPEED_1M8
	cpi TEMP1,0x0A          ;timer is active, WGM12,CS11 (timer/8)
#endif
#ifdef SPEED_7M
	cpi TEMP1,0x0A          ;timer is active, WGM12,CS11 (timer/8)
#endif
#ifdef SPEED_11M
	cpi TEMP1,0x0B          ;timer is active, WGM12,CS11 and CS10 (timer/64)
#endif
#ifdef SPEED_14M
	cpi TEMP1,0x0B          ;timer is active, WGM12,CS11 and CS10 (timer/64)
#endif
#ifdef SPEED_16M
	cpi TEMP1,0x0B          ;timer is active, WGM12,CS11 and CS10 (timer/64)
#endif
#ifdef SPEED_18M
	cpi TEMP1,0x0B          ;timer is active, WGM12,CS11 and CS10 (timer/64)
#endif
	breq WAV80              ;WAVE already oprating
WAV70:
;WAVE neu starten

#if (CPU==32)
	clr TEMP1
	out TCCR1B,TEMP1        ;first stop timer
	out TCNT1H,TEMP1
	out TCNT1L,TEMP1        ;reset TCNT1
	out OCR1AH,WAVELOH    	;enter Low-value first time
	out OCR1AL,WAVELOL

	sbi DDRD,PMO1          	;output
	cbi PORTD,PMO1         	;start Low
	ldi TEMP1,0xC0          ;set OC1A/B on next compare match
	out TCCR1A,TEMP1

  #ifdef SPEED_1M8
	ldi TEMP1,0x0A          ;divide /8,reset at OC1A  -- CTC(WGM12 and CS11)
  #endif
  #ifdef SPEED_7M
	ldi TEMP1,0x0A          ;divide /8,reset at OC1A  -- CTC(WGM12 and CS11)
  #endif
  #ifdef SPEED_11M
	ldi TEMP1,0x0B          ;divide /64,reset at OC1A  -- CTC(WGM12,CS11,CS10)
  #endif
  #ifdef SPEED_14M
	ldi TEMP1,0x0B          ;divide /64,reset at OC1A  -- CTC(WGM12,CS11,CS10)
  #endif
  #ifdef SPEED_16M
	ldi TEMP1,0x0B          ;divide /64,reset at OC1A  -- CTC(WGM12,CS11,CS10)
  #endif
  #ifdef SPEED_18M			;should be not allowed, but for precaution here
	ldi TEMP1,0x0B          ;divide /64,reset at OC1A  -- CTC(WGM12,CS11,CS10)
  #endif
	out TCCR1B,TEMP1

	in TEMP1,TIMSK        	;activate OC1A interrupt
	ori TEMP1,(1<<OCIE1A)  
	out TIMSK,TEMP1

	sbr XFLAGS,m_TIM1BUSY
#else
	clr TEMP1
	sts TCCR1B,TEMP1        ;first stop timer
	sts TCNT1H,TEMP1
	sts TCNT1L,TEMP1        ;reset TCNT1
	sts OCR1AH,WAVELOH     	;enter Low-value first time
	sts OCR1AL,WAVELOL

  #if ((CPU==168) || (CPU==328) || (CPU==3244))
	sbi DDRB,PMO1          	;output
	cbi PORTB,PMO1         	;start Low
  #endif
  #if ((CPU==644) || (CPU==1284))
	sbi DDRD,PMO1          	;output
	cbi PORTD,PMO1         	;start Low
  #endif

  #ifdef SPEED_1M8
	ldi TEMP1,0x0A          ;divide /8,reset at OC1A  -- CTC(WGM12 and CS11)
  #endif
  #ifdef SPEED_7M
	ldi TEMP1,0x0A          ;divide /8,reset at OC1A  -- CTC(WGM12 and CS11)
  #endif
  #ifdef SPEED_11M
	ldi TEMP1,0x0B          ;divide /64,reset at OC1A  -- CTC(WGM12,CS11,CS10)
  #endif
  #ifdef SPEED_14M
	ldi TEMP1,0x0B          ;divide /64,reset at OC1A  -- CTC(WGM12,CS11,CS10)
  #endif
  #ifdef SPEED_16M
	ldi TEMP1,0x0B          ;divide /64,reset at OC1A  -- CTC(WGM12,CS11,CS10)
  #endif
  #ifdef SPEED_18M
	ldi TEMP1,0x0B          ;divide /64,reset at OC1A  -- CTC(WGM12,CS11,CS10)
  #endif
	sts TCCR1B,TEMP1

	ldi TEMP1,0xC0          ;set OC1A on next compare match
	sts TCCR1A,TEMP1
	ldi TEMP1,(1<<OCIE1A)   ;type independent
	sts TIMSK1,TEMP1

	sbr XFLAGS,m_TIM1BUSY
#endif

WAV80:  
	sei
	ld TOSL,X+				;DROP
	ld TOSH,X+
	ret


WAVEHI:  ;  ( x -- )  ***********************************************
;only sets value, doesn't start timer.
;WAVE-Parameter as Register
;Emegency-Default if WAVELO >= actual period
;WAVELO is obtained by inversion HI/LO
;********************************************************************
;check if Timer already WAVE operation
	cli
	tst TOSH
	brne WVH10
	cpi TOSL,8
	brcc WVH10
	ldi TOSL,8				;MIN!!,else interrupt generates permanent HIGH out
WVH10:
#if (CPU==32)
	in TEMP1,TCCR1A
#else
	lds TEMP1,TCCR1A
#endif
	andi TEMP1,0xBF          ;don't care Bit6=COM1A0, is changing
	cpi TEMP1,0x80
	brne WVH20              ;only COM1A1 set = WAVE active, PWGM12->CTC mode
#if (CPU==32)
	in TEMP1,TCCR1B
#else
	lds TEMP1,TCCR1B
#endif
	cpi TEMP1,0x0B			;WGM12,CS11 and CS10
	breq WVH40              ;WAVE already operating
	cpi TEMP1,0x0A			;WGM12,CS11
	breq WVH40              ;WAVE already operating
WVH20:
;if WAVE not active: only keep WAVEHI in mind
	sts WAVEHIL,TOSL
	sts WAVEHIH,TOSH
	sei
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 

WVH40:
	mov r24,WAVELOL
	mov r25,WAVELOH
	lds TEMP1,WAVEHIL
	lds TEMP2,WAVEHIH
	add r24,TEMP1
	adc r25,TEMP2			;r24,r25= actual periode time
	sts WAVEHIL,r24
	sts WAVEHIH,r25         ;keep in mind for emergency default
	sub r24,TOSL            ;calculate new WAVELO
	sbc r25,TOSH
	brcs WVH50              ;emergency default of would be WAVELO <= 0
	brne WVH60              ;continue normally, if new WAVELO > 0
WVH50:
;set emergency default: WAVEHI-=1, WAVELO=1
	ldi r24,8
	clr r25
	sub TOSL,r24
	sbc TOSH,r25
	sts WAVEHIH,TOSH
	sts WAVEHIL,TOSL
	clr WAVELOH
	ldi r25,8				;min value for interrupt
	mov WAVELOL,r25
	rjmp WVH80
WVH60:
;normal weiter
	sts WAVEHIL,TOSL
	sts WAVEHIH,TOSH
	mov WAVELOL,r24
	mov WAVELOH,r25
WVH80:  
	sei
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


;WAVE Interrupt Routine

waveInt:    ;********************************************************
;WAVEx in SRAM
;********************************************************************
#if (CPU==32)
	in SAVE_SREG,SREG
	in INTREG,TCCR1A
	sbrs INTREG,COM1A0
	rjmp WVI40
;set:now  High-phase active. Set termination time, set"Clear OC1A"
	lds INTREG,WAVEHIH
	out OCR1AH,INTREG
	lds INTREG,WAVEHIL
	out OCR1AL,INTREG

	in INTREG,TCCR1A
	cbr INTREG,(1<<COM1A0) ;m_COM1A0
	out TCCR1A,INTREG
	out SREG,SAVE_SREG
	reti
WVI40:
;set:now  High-phase active. Set termination time, activate "Set OC1A"
	out OCR1AH,WAVELOH
	out OCR1AL,WAVELOL

	in INTREG,TCCR1A
	sbr INTREG,(1<<COM1A0) ;m_COM1A0
	out TCCR1A,INTREG
	out SREG,SAVE_SREG
	reti
#else
	in SAVE_SREG,SREG
	lds INTREG,TCCR1A
	sbrs INTREG,COM1A0
	rjmp WVI40
;set:now  High-phase active. Set termination time, set"Clear OC1A"
	lds INTREG,WAVEHIH
	sts OCR1AH,INTREG
	lds INTREG,WAVEHIL
	sts OCR1AL,INTREG

	lds INTREG,TCCR1A
	cbr INTREG,(1<<COM1A0) ;m_COM1A0
	sts TCCR1A,INTREG
	out SREG,SAVE_SREG
	reti
WVI40:
;set:now  High-phase active. Set termination time, activate "Set OC1A"
	sts OCR1AH,WAVELOH
	sts OCR1AL,WAVELOL

	lds INTREG,TCCR1A
	sbr INTREG,(1<<COM1A0) ;m_COM1A0
	sts TCCR1A,INTREG
	out SREG,SAVE_SREG
	reti
#endif

PWM1:  ;  ( x -- )  ************************************************
; OC1A is PB1(28pin DIL) or PD5(40pin DIL)
; atMega32U4:OC1B is PB6
; configure PWM1 mode, prescaler and pulse  ratio
; plausible simple setup(H-Arg=0)
; switch PWM1 OFF only by call of IP,IZ,OH,OL
;********************************************************************
	mov r24,TOSH
	cpi r24,0x0F
	brcs PM130				;8bit resolution,with prescaler
//10 bit resolution with prescaler
	swap r24
	andi r24,0x0F			;r24 is prescaler value 1...5
	cpi r24,6
	brcs PM110
	ldi r24,5
PM110:
	andi TOSH,0x0F	
	cpi TOSH,4
	brcs PM120
	ldi TOSH,3				;max TOS range 0...0x3FF	
PM120:	
#if (CPU==3244)
	ldi r25,(1<<COM1B1) | (1<<WGM10) | (1<<WGM11)
#else
	ldi r25,(1<<COM1A1) | (1<<WGM10) | (1<<WGM11)
#endif
	rjmp PM160
PM130:
//8 bit resolution with prescaler
	andi r24,0x0F
	tst r24
	brne PM140
	ldi r24,1				;no prescaler
PM140:
	cpi r24,6
	brcs PM150
	ldi r24,5				;r24 is prescaler value 1...5
PM150:
	clr TOSH
#if (CPU==3244)
	ldi r25,(1<<COM1B1) | (1<<WGM10)
#else
	ldi r25,(1<<COM1A1) | (1<<WGM10) 
#endif
PM160:
#if (CPU==32)
	out TCCR1A,r25
	out OCR1AH,TOSH
	out OCR1AL,TOSL
#else
  #if(CPU==3244)
	sts TCCR1A,r25
	sts OCR1BH,TOSH
	sts OCR1BL,TOSL
  #else
	sts TCCR1A,r25
	sts OCR1AH,TOSH
	sts OCR1AL,TOSL
  #endif
#endif

#if ((CPU==168) || (CPU==328))
	sbi DDRB,PMO1          	;PB1 output
	sts TCCR1B,r24
#endif
#if ((CPU==644) || (CPU == 1284))
	sbi DDRD,PMO1          	;PD5 output
	sts TCCR1B,r24
#endif
#if (CPU==3244)
	sbi DDRB,PMO1          	;PB6 output
	sts TCCR1B,r24
#endif
#if (CPU==32)
	sbi DDRD,PMO1          ;PD5 output
	out TCCR1B,r24
#endif
	sbr XFLAGS,m_TIM1BUSY
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 


#if (CPU==3244)
PWM3:  ;  ( x -- )  *************************************************
;OC3A is PC6
;configure PWM3A mode, prescaler and pulse  ratio
;plausible simple setup(H-Arg=0)
;switch PWM3 OFF only by call of IP,IZ,OH,OL
;********************************************************************
	mov r24,TOSH
	cpi r24,0x0F
	brcs PM330				;8bit resolution,with prescaler
//10 bit resolution with prescaler
	swap r24
	andi r24,0x0F			;r24 is prescaler value 1...5
	cpi r24,6
	brcs PM310
	ldi r24,5
PM310:
	andi TOSH,0x0F	
	cpi TOSH,4
	brcs PM320
	ldi TOSH,3				;max TOS range 0...0x3FF	
PM320:	;
	ldi r25,(1<<COM3A1) | (1<<WGM30) | (1<<WGM31)
	rjmp PM360
PM330:
//8 bit resolution with prescaler
	andi r24,0x0F
	tst r24
	brne PM340
	ldi r24,1				;no prescaler
PM340:
	cpi r24,6
	brcs PM350
	ldi r24,5				;r24 is prescaler value 1...5
PM350:
	clr TOSH
	ldi r25,(1<<COM3A1) | (1<<WGM30) 
PM360:
	sts TCCR3A,r25
	sts OCR3AH,TOSH
	sts OCR3AL,TOSL
	sbi DDRC,PMO3			;PC6 output
	sts TCCR3B,r24
	sbr XFLAGS,m_PWMBUSY
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+
	ret

#else

PWMM2:  ; ( x -- ) **************************************************
; OC2A is pin PD7 (mega32,644,1284)
; OC2B is pin PD3( mega168,328) ,
; configure PWM2 prescaler and pulse ratio
; plausible simple setup(H-Arg=0)
; switch PWM2 OFF only by call of IP,IZ,OH,OL
;********************************************************************
  #if (CPU==32)
	in r25,TCCR2
	andi r25,(1<<WGM20)     ;check PWM2=WGM20
	brne PM210              ;jump if Timer2 is already in PWM mode
	clr r25
	out TCCR2,r25           ;Disable Timer2
  #endif

PM210:
;Bits 8,9,10=prescaler setup
	cpi TOSH,8
	brcc PM280              ;invalid prescaler
	tst TOSH
	brne PM220				;no entry, if 0 preset
	ldi TOSH,1              ;simple frequency setup 0 sets 0x100, too
PM220:
;set PWM-High phase
  #if (CPU==32)
	out OCR2,TOSL
  #endif
  #if ((CPU==644) || (CPU==1284))
	sts OCR2A,TOSL
  #endif
  #if ((CPU==168) || (CPU==328))
	sts OCR2B,TOSL
  #endif

  #if (CPU==32)
	ori TOSH,0x60           		;PWM2=WGM20 und COM21 setzen
	out TCCR2,TOSH					;=Phase correct PWM
  #endif
  #if ((CPU==644) || (CPU==1284))
	ldi r25,(1<<COM2A1|1<<WGM20)  	;PWM2=WGM20 und COM21 setzen
	sts TCCR2A,r25			 		;=Phase correct PWM
	sts TCCR2B,TOSH	
  #endif	       
  #if ((CPU==168) || (CPU==328))
	ldi r25,(1<<COM2B1|1<<WGM20)  	;PWM2=WGM20 und COM21 setzen
	sts TCCR2A,r25			 		;=Phase correct PWM
	sts TCCR2B,TOSH		
  #endif

  #if (CPU==32)
	sbi DDRD,PMO2           ;output
  #endif
  #if ((CPU==644) || (CPU==1284))
	sbi DDRD,PMO2           ;output
  #endif
  #if ((CPU==168) || (CPU==328))
	sbi DDRD,PMO2           ;output
  #endif
	sbr XFLAGS,m_PWMBUSY
PM280:
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+		
	ret 

#endif ;(CPU==3244) or not


INITSPI:   ;( b -- )  ***********************************************
;High Nibble MODE (0,1,2,3), Low nibbble SPEED (0-7)
;TOSH is ignored
;********************************************************************
;configure SCK MOSI DDR as output, MISO as input
;check /SS . If input, then activate pull-up 

#if ((CPU==168) || (CPU==328))
//PB2=/SS, PB3=MOSI, PB4=MISO, PB5=SCK
	in TOSH,DDRB
	ori TOSH,0x28		;configure PB3&5 as output
	andi TOSH,0xEF		;configure PB4 as input
	out DDRB,TOSH

	sbrc TOSH,2			;PB2 is /SS
	rjmp ISP10			;if it is input,it must be configurend with Pull-UP
	in TOSH,PORTB		;if it is output:no change
	ori TOSH,4
	out PORTB,TOSH
#endif
#if ((CPU==644) || (CPU==1284))
//PB4=/SS, PB5=MOSI, PB6=MISO, PB7=SCK
	in TOSH,DDRB
	ori TOSH,0xA0		;configure PB7&5 as output	
	andi TOSH,0xBF		;configure PB6 as input
	out DDRB,TOSH

	sbrc TOSH,4			;PB4 is /SS
	rjmp ISP10 			;if it is input,it must be configurend with Pull-UP
	in TOSH,PORTB		;if it is output:no change
	ori TOSH,0x10
	out PORTB,TOSH
#endif
#if (CPU==3244)
//PB0=/SS, PB1=SCK, PB2=MOSI, PB3=MISO
	in TOSH,DDRB
	ori TOSH,6			;configure PB1&2 as output	
	andi TOSH,0xF7		;configure PB3 as input
	out DDRB,TOSH

	sbrc TOSH,0			;PB0 is /SS
	rjmp ISP10 			;if it is input,it must be configurend with Pull-UP
	in TOSH,PORTB		;if it is output:no change
	ori TOSH,1
	out PORTB,TOSH
#endif
ISP10:
;enable SPI, Master, Mode, clock rate
	clr TOSH
	mov r24,TOSL
	andi r24,0x0F
	cpi r24,7
	brcs ISP20				;valid range 0...6
	jmp parameterErr
ISP20:
	inc r24					;now range 1..7
	cpi r24,3
	brcs ISP30
	sbrc r24,0
	ldi TOSH,1
ISP30:
	out SPSR,TOSH			;set SPI2X bit, double speed	

	lsr r24
	com r24					;invert speed order 0->3, 1->2, 2->1, 3->0
	andi r24,3				;separate bits 0,1
	mov TOSH,TOSL.
	andi TOSH,0xC0			;check invalid range
	brne SPI90

	lsr TOSL
	lsr TOSL				;bits 5,4 TOSL ->bits 3,2 TOSL now (mode0,1,2,3)			
	andi TOSL,0x0C
	or TOSL,r24				;TOSH bits 3,2,1,0 configuration now
	ori TOSL,0x50			;set SPE(enable SPI) and set MASTER bit
	out SPCR,TOSL			;final entry

	sbr XFLAGS,m_SPIBUSY

	ld TOSL,X+				;DROP
	ld TOSH,X+
	in r24,SPSR				;as precaution, clear interrupt bit
	in r24,SPDR				;else previous MISO IN may be returned
 	ret


SPI:   ;( b -- b )  *************************************************
;exclusively 8 bit data, TOSH is ignored
;********************************************************************
	sbrs XFLAGS,SPIBUSY
	rjmp SPI90
;must be configured as MASTER !


	out SPDR,TOSL	;data SPDR (MOSI out)
SPI10:
	in TOSL,SPSR
	sbrs TOSL,SPIF
	rjmp SPI10
;	read MISO in data
	clr TOSH
	in TOSL,SPDR	
 	ret
SPI90: 
	jmp parameterErr

; ###################################################################

; end of Kernel code

	.ORG	KERNELTOP
	  .DW 0xFFFF	;trigger assembler for check of kernel code overflow



;####################################################################
