;********************************************************************
;AVISE5.ASM
;copied from AVISE43.ASM  05.Feb.2023 and essentially modified
;Source AVISE Forth Interpreter with Subroutine threaded user code
;16 Bit Stack, 16bit Token
;internal Flash used for storage of user function token code
;User Symbol Tabelle in CPU-EEPROM
;Kernel Symbol Table and Kernel Operator code in AV5K.ASM
;Kernel Operators for hardware spezific  I/O in AV5S.ASM
;USB specific kernel files in AV5U.ASM
;********************************************************************
.NOLIST

#define HW_VERSION 5
#define REVNUM 9

#define CPU 168		;MEGA168 (16kFlash 1kRAM,512 Byte >EEPROM)
;#define CPU 328         ;MEGA328 (2k RAM,32KFlash)/
;#define CPU 644		;MEGA644P (2k RAM,64KFlash)
;#define CPU 1284		;MEGA1284P (4k RAM,128KFlash)
;#define CPU 3244		;MEGA32U4 (2k RAM,32KFlash)
;#define CPU 32         ;MEGA32 (2k RAM,32KFlash)

#if ((CPU != 32) && (CPU != 3244))
#define SPEED_1M8		;special:1.8432 MHz (starts w.921kHz,see XFLAGS,HISPEED)
;#define SPEED_7M		;special:7.3728 MHz (starts w.921kHz,see XFLAGS,HISPEED)
#endif
;#define SPEED_14M		;14.7456 MHz
;#define SPEED_16M		;only option for ATmega32U4 and ARDUINO UNO
#if (CPU != 32)
;#define SPEED_18M		;18.432 MHz
#endif

#if (CPU==168) 			;ATmega168
	.INCLUDE "m168defmod.inc"
//PAGESIZE = 64 WORD Segments
#define FLASHPAGE  		64 ;=PAGESIZE
#define FLASH           16
#define USERFLASH       0x1F80  ;in WORDS - must be on page limit!
#define KERNELTOP       0x1500  ;in WORDS - must be on page limit!   
#define RAM             0x400   ;1k BYTES
#define INTEEP          0x200   ;512 BYTES
#define ANAIN           6

 #ifdef SPEED_16M
#define ARDUINO_UNO   ;Option, fake 115200
 #endif
#endif

#if (CPU==328)                	;ATmega328
	.INCLUDE "m328Pdefmod.inc"
//PAGESIZE = 64 WORD Segments
#define FLASHPAGE  		64		;=PAGESIZE
#define FLASH           32
#define USERFLASH       0x3F00  ;in WORDS - must be on page limit!
#define KERNELTOP       0x1500  ;in WORDS - must be on page limit!
#define RAM             0x800   ;2k BYTES
#define INTEEP          0x400   ;1k BYTES
#define ANAIN           6

 #ifdef SPEED_16M
#define ARDUINO_UNO   ;Option, fake 115200
 #endif
#endif

#if (CPU==644)                  ;ATmega644
	.INCLUDE "m644Pdefmod.inc"
//PAGESIZE = 128 WORD Segments
#define FLASHPAGE		128		;=PAGESIZE
#define FLASH           64
#define USERFLASH       0x7E00  ;in WORDS - must be on page limit!
#define KERNELTOP       0x1600  ;in WORDS - must be on page limit!
#define RAM             0x1000	;4k BYTES
#define INTEEP          0x800	;2k BYTES
#define ANAIN           8
#endif

#if (CPU==1284)                  ;ATmega1284
	.INCLUDE "m1284Pdefmod.inc"
//PAGESIZE = 128 WORD Segments
#define FLASHPAGE 		128		;=PAGESIZE
#define FLASH           128
#define USERFLASH       0xFE00  ;in WORDS - must be on page limit!
#define KERNELTOP       0x1600  ;in WORDS - must be on page limit!
#define RAM             0x4000	;16k BYTES
#define INTEEP          0x1000	;4k BYTES
#define ANAIN           8
#endif

#if (CPU==3244)                	;ATmega328
	.INCLUDE "m32U4defmod.inc"
//PAGESIZE = 64 WORD Segments
#define FLASHPAGE  		64		;=PAGESIZE
#define FLASH           32
#define USERFLASH       0x3F00  ;in WORDS
#define KERNELTOP       0x1A00  ;in WORDS
#define RAM             0x800   ;2k BYTES
#define INTEEP          0x400   ;1k BYTES
#define ANAIN           6

//#define LEONARDO				;only difference LED pin,polarity, PID
;else implicitly:MICRO
#endif

#if (CPU==32)                   ;ATmega32
	.INCLUDE "m32defmod.inc"
//PAGESIZE = 64 WORD Segments
#define FLASHPAGE  		64;=PAGESIZE
#define FLASH           32
#define USERFLASH       0x3F00  ;in WORDS
#define KERNELTOP       0x1500  ;in WORDS
#define RAM             0x800   ;2k	BYTES
#define INTEEP          0x400   ;1k	BYTES
#define ANAIN           8
#endif

	.INCLUDE "AV5H.ASM"

;####################################################################

	.LIST

	.CSEG

;offsets in WORDS for Avra Assembler
;all Interrupts in jmp format (2 WORDS)

 #if ((CPU==168)||(CPU==328))
    .ORG 0
	rjmp START
    .ORG  INT0addr              ; External Interrupt Request 0
	rjmp countInt0
    .ORG  INT1addr              ; External Interrupt Request 1
	reti
    .ORG  PCI0addr              ; Pin Change Interrupt Request 0
	reti
    .ORG  PCI1addr              ; Pin Change Interrupt Request 0
	reti
    .ORG  PCI2addr              ; Pin Change Interrupt Request 1
	rjmp sleepInt
    .ORG  WDTaddr               ; Watchdog Time-out Interrupt
	reti
    .ORG  OC2Aaddr              ; Timer/Counter2 Compare Match A
	reti
    .ORG  OC2Baddr              ; Timer/Counter2 Compare Match B
	reti
    .ORG  OVF2addr              ; Timer/Counter2 Overflow
	reti
    .ORG  ICP1addr              ; Timer/Counter1 Capture Event
	reti
    .ORG  OC1Aaddr              ; Timer/Counter1 Compare Match A
	jmp waveInt
    .ORG  OC1Baddr              ; Timer/Counter1 Compare Match B
	reti
    .ORG  OVF1addr              ; Timer/Counter1 Overflow
	reti
    .ORG  OC0Aaddr              ; TimerCounter0 Compare Match A
	reti
    .ORG  OC0Baddr              ; TimerCounter0 Compare Match B
	reti
    .ORG  OVF0addr              ; Timer/Counter0 Overflow
	jmp timer0Int
    .ORG  SPIaddr               ; SPI Serial Transfer Complete
	reti
    .ORG  URXCaddr              ; USART Rx Complete
	rjmp cmdSrxInt
    .ORG  UDREaddr              ; USART, Data Register Empty
	reti
    .ORG  UTXCaddr              ; USART Tx Complete
	reti
    .ORG  ADCCaddr               ; ADC Conversion Complete
	reti
    .ORG  ERDYaddr              ; EEPROM Ready
	reti
    .ORG  ACIaddr               ; Analog Comparator
	reti
    .ORG  TWIaddr               ; Two-wire Serial Interface
	reti
    .ORG  SPMRaddr              ; Store Program Memory Read
	reti

#endif ;mega168,mega328P


#if ((CPU==644)||(CPU==1284))
	.ORG 0
	rjmp START
    .ORG INT0addr       ;External Interrupt0 Vector Address
	rjmp countInt0
    .ORG INT1addr       ;External Interrupt1 Vector Address
	reti
    .ORG INT2addr       ;External Interrupt2 Vector Address
	rjmp countIntX
    .ORG PCI0addr       ;Pin Change Interrupt0 Vector Address
	reti
    .ORG PCI1addr       ;Pin Change Interrupt1 Vector Address
	reti
    .ORG PCI2addr       ;Pin Change Interrupt2 Vector Address
	reti
    .ORG PCI3addr       ;Pin Change Interrupt3 Vector Address
	rjmp sleepInt
    .ORG WDTaddr        ;Watch Dog Timer Vector Address
	reti
    .ORG OC2Aaddr       ;Output Compare2A Interrupt Vector Address
	reti
    .ORG OC2Baddr       ;Output Compare2B Interrupt Vector Address
	reti
    .ORG OVF2addr       ;Overflow2 Interrupt Vector Address
	reti 
    .ORG ICP1addr       ;Input Capture1 Interrupt Vector Address
	reti
    .ORG  OC1Aaddr      ;Output Compare1A Interrupt Vector Address
	jmp waveInt
    .ORG OC1Baddr       ;Output Compare1B Interrupt Vector Address
	reti
    .ORG OVF1addr       ;Overflow1 Interrupt Vector Address
	reti
    .ORG OC0Aaddr       ;Output Compare0A Interrupt Vector Address
	reti
    .ORG OC0Baddr       ;Output Compare0B Interrupt Vector Address
	reti
    .ORG OVF0addr       ;Overflow0 Interrupt Vector Address
	rjmp timer0Int
    .ORG SPIaddr        ;SPI Interrupt Vector Address
	reti
    .ORG URXC0addr      ;USART0 Receive Complete Interrupt Vector Address
	rjmp cmdSRXint
    .ORG UDRE0addr      ;USART0 Data Register Empty Interrupt Vector Address
	reti 
    .ORG UTXC0addr      ;USART0 Transmit Complete Interrupt Vector Address
	reti
    .ORG ACIaddr        ;Analog Comparator
	reti
    .ORG ADCCaddr       ;ADC Conversion Complete Handle
	reti
    .ORG ERDYaddr       ;EEPROM Write Complete Handle
	reti
    .ORG TWIaddr        ;I2C Interrupt Vector Address
	reti
    .ORG SPMRaddr       ;Store Program Memory Ready Interrupt Vector Address
	reti
    .ORG URXC1addr      ;USART1 Receive Complete Interrupt Vector Address
	rjmp US1SRXint
    .ORG UDRE1addr      ;USART1 Data Register Empty Interrupt Vector Address
	reti				   ;wird u.U. nicht aktiviert
    .ORG UTXC1addr      ;USART1 Transmit Complete Interrupt Vector Address
	reti
;ATmega1284: some unused entries omitted					
#endif ;maga644,mega1284


 #if (CPU==3244)
   .ORG 0
	rjmp START
    .ORG INT0addr			;External Interrupt Request 0
	rjmp countInt0
	.ORG INT1addr			;External Interrupt Request 1
	rjmp countIntX
	.ORG INT2addr			;External Interrupt Request 2
	reti
	.ORG INT3addr			;External Interrupt Request 3
	reti
	.ORG INT6addr			;External Interrupt Request 6
	reti
    .ORG PCI0addr			;Pin Change Interrupt Request 0
	reti
    .ORG USB_GENaddr		;USB General Interrupt Request
	rjmp ISRGEN          	
    .ORG USB_COMaddr		;USB Endpoint/Pipe Interrupt Communication Request
    rjmp ISRCOM            	;COMmunication Interrupt   	
    .ORG WDTaddr			;Watchdog Time-out Interrupt
	reti
    .ORG ICP1addr			;Timer/Counter1 Capture Event
	reti
    .ORG OC1Aaddr			;Timer/Counter1 Compare Match A
	jmp waveInt
    .ORG OC1Baddr			;Timer/Counter1 Compare Match B
	reti
    .ORG OC1Caddr			;Timer/Counter1 Compare Match C
	reti
    .ORG OVF1addr			;Timer/Counter1 Overflow
	reti
    .ORG OC0Aaddr			;Timer/Counter0 Compare Match A
	reti
   .ORG OC0Baddr			;Timer/Counter0 Compare Match B
	reti
    .ORG OVF0addr			;Timer/Counter0 Overflow
	jmp timer0Int
    .ORG SPIaddr			;SPI Serial Transfer Complete
	reti
    .ORG URXC1addr			;USART1, Rx Complete
	rjmp US1SRXint
    .ORG UDRE1addr			;USART1 Data register Empty
	reti
   .ORG UTXC1addr			;USART1, Tx Complete
	reti
    .ORG ACIaddr			;Analog Comparator
	reti
    .ORG ADCCaddr			;ADC Conversion Complete
	reti
    .ORG ERDYaddr			;EEPROM Ready
	reti
    .ORG ICP3addr			;Timer/Counter3 Capture Event
	reti
    .ORG OC3Aaddr			;Timer/Counter3 Compare Match A
	reti
    .ORG OC3Baddr			;Timer/Counter3 Compare Match B
	reti
    .ORG OC3Caddr			;Timer/Counter3 Compare Match C
	reti
    .ORG OVF3addr			;Timer/Counter3 Overflow
	reti
   .ORG TWIaddr				;2-wire Serial Interface
	reti
    .ORG SPMRaddr			;Store Program Memory Read
	reti
    .ORG OC4Aaddr			;Timer/Counter4 Compare Match A
	reti
    .ORG OC4Baddr			;Timer/Counter4 Compare Match B
	reti
    .ORG OC4Daddr			;Timer/Counter4 Compare Match D
	reti
    .ORG OVF4addr			;Timer/Counter4 Overflow
	reti
    .ORG TIMER4_FPFaddr		;Timer/Counter4 Fault Protection Interrupt
	reti

#endif ;mega32U4


#if (CPU==32)				;ATmega32
   .ORG 0
	rjmp START
   .ORG INT0addr
	rjmp countInt0 	  		;External Interrupt0 Vector Address
   .ORG INT1addr
	rjmp countIntX			;External Interrupt1 Vector Address
   .ORG INT2addr
	rjmp sleepInt			;External Interrupt2 Vector Address
   .ORG OC2addr
	reti              		;Output Compare2 Interrupt Vector Address
   .ORG OVF2addr
	reti           			;Overflow2 Interrupt Vector Address
   .ORG ICP1addr
	reti              		;Input Capture1 Interrupt Vector Address
   .ORG OC1Aaddr
	jmp waveInt       		;Output Compare1A Interrupt Vector Address
   .ORG OC1Baddr
	reti               		;Output Compare1B Interrupt Vector Address
   .ORG OVF1addr
	reti               		;Overflow1 Interrupt Vector Address
   .ORG OC0addr
	reti               		;Output Compare0 Interrupt Vector Address
   .ORG OVF0addr
	jmp timer0Int     		;Overflow0 Interrupt Vector Address
   .ORG SPIaddr
	reti               		;SPI Interrupt Vector Address
   .ORG URXCaddr
	jmp cmdSRXint   		;USART Receive Complete Interrupt Vector Address
   .ORG UDREaddr
	reti            		;USART TX Data Register Empty Interrupt Vector Addr
   .ORG UTXCaddr
	reti            		;USART Transmit Register Empty Interrupt Vector Addr
   .ORG ADCCaddr
	reti
   .ORG ERDYaddr
	reti
   .ORG ACIaddr
	reti
   .ORG TWSIaddr
	reti
   .ORG SPMRaddr
	reti

#endif

   .ORG 0x80		;WORD !!

HISTRING:
	.DB CRR,"Avise v5.",REVNUM+0x30,"  ATmega",0
TYPESTR:
#if (CPU==168)
	.DB "168  ",0
#endif
#if (CPU==328)
	.DB "328  ",0
#endif
#if (CPU==644)
	.DB "644  ",0
#endif
#if (CPU==1284)
	.DB "1284  ",0,0
#endif
#if (CPU==3244)
	.DB "32U4  ",0,0
#endif
#if (CPU==32)
	.DB "32  ",0,0
#endif
SPEEDSTR:
#ifdef SPEED_1M8
	.DB "1.84MHz",CRR,0,0
#endif
#ifdef SPEED_7M
	.DB "7.37MHz",CRR,0,0
#endif
#ifdef SPEED_11M
	.DB "11.059MHz",CRR,0,0
#endif
#ifdef SPEED_14M
	.DB "14.7MHz",CRR,0,0
#endif
#ifdef SPEED_16M
	.DB "16.0MHz",CRR,0,0
#endif
#ifdef SPEED_18M
	.DB "18.4MHz",CRR,0,0
#endif
AUTOSTRING:
	.DB "AUTOEXE:",0,0
TOKENSTR:
	.DB "unknown",0
MEMSTR:
	.DB "memory-",0             ;'page ',0
STACKSTR:
	.DB "stack-",0,0
RUNSTR:
	.DB "runtime-",0,0
SYNTAXSTR:
	.DB "syntax-",0
ERRSTR:
	.DB "error~",0,0
PARAMSTR:
	.DB "invalid parameter~",0,0
COMPOSTR:
	.DB "compile only~",0
ABORTSTR:
	.DB "aborted",0
OKSTR:
	.DB "  ok",0,0
CMPISTR:
	.DB "  cp",0,0
KERNELSTR:
	.DB CRR,"kernel ops:",CRR,0
TOSSTR:
	.DB "<=TOS  VA:",0,0
VARSTR:
	.DB " VARs:",0,0
FLUSHSTR:
	.DB "flushed Stack:",0,0
#if (CPU==3244)
NEWSTR:
	.DB " new_",0
OLDSTR:
	.DB " old_",0
VIDSTR:
	.DB "VID=",0,0
PIDSTR:			
	.DB "PID=",0,0
WORDSTR:	
	.DB " -- oonfirm w.'Y'",0
#else 
EEPROMSTR:
	.DB CRR,"EEProm:",0,0
CBUFSTR:
	.DB CRR,"temp.SRAM:",CRR,0,0
DECIMALSTR:
	.DB "(decimalWORDS) S:",0
#endif

BITMASK:								;convert bit number into byte mask
	.DB 1,2,4,8,0x10,0x20,0x40,0x80


START:  ; *******************************************************************
	cli
	ldi ZL,LOW(RETURNSTACK)
	out SPL,ZL
	ldi ZL,HIGH(RETURNSTACK)
	out SPH,ZL
	
	wdr
#if (CPU==32)
;re-init Watchdog 
	ldi r24,0x1E            ;double step init necessary
	ldi r25,0xE             ;1sec timeout
	out WDTCR,r24
	out WDTCR,r25
#else
;re-init Watchdog 
	ldi r24,0x1E            ;double step init necessary
	ldi r25,0xE             ;1sec timeout
	sts WDTCSR,r24
	sts WDTCSR,r25
#endif

;clear all SRAM variables to ZERO 
;especially inclusive all variables and ARRAY
	ldi ZH,HIGH(SRAMBOT)		
	ldi ZL,LOW(SRAMBOT)
	clr r25
STT10:  
	st Z+,r25				;clears USB buffers too
	cpi ZH,HIGH(INIMAX)
	brcs STT10

;prestore seed for RANDOM
	ldi r25,0x7A
	sts RANDOMH,r25
	ldi r25,0xB7
	sts RANDOML,r25

	clr LEDCNT
	ldi r25,100
	mov MS100CNT,r25

#ifdef SPEED_1M8			;generally start with 921 kHz.
	ldi r24,0x80			
	ldi r25,1				;2 step clock prescaler init procedure
	sts CLKPR,r24			;only write CLKPCE bit
	sts CLKPR,r25			;set division/2
#endif

#ifdef SPEED_7M				;generally start with 921 kHz.
	ldi r24,0x80			
	ldi r25,3				;2 step clock prescaler init procedure
	sts CLKPR,r24			;only write CLKPCE bit
	sts CLKPR,r25			;set division/8
#endif

;aactivate all ports as "default"
;i.e. inputs with pull-up
	clr r25
	ldi r24,0xFF
	out DDRB,r25			;input
	out PORTB,r24			;pull-up
	out DDRC,r25
	out PORTC,r24
	out DDRD,r25
	out PORTD,r24
  #if ((CPU==644)||(CPU==1284))
	out DDRA,r25
	out PORTA,r24 			;if analog IN is started, special config
  #endif
  #if (CPU==3244)
	out DDRE,r25
	out PORTE,r24 
	out DDRF,r25
	out PORTF,r24 			;if analog IN is started, special config
  #endif

;init variables and global control flags
	ldi ZL,LOW(DEFNUMB)
	ldi ZH,HIGH(DEFNUMB)
	call addressIntEep
	in r24,EEDR
	cpi r24,0xFF
	brne STD20
;virgin EEProm and general default is "DECIMAL" number base
;can be activated with "0xFF 0xD WREEP" and perform CPU reset
#if ((CPU==168) || (CPU==328))
 	ldi AVIFLAGS,m_VREF0	;assume decimal number base as default
#else
 	ldi AVIFLAGS,m_VREF0	;assume decimal number base as default
#endif
	rjmp STD30
STD20:
;alternative is "HEX" number base
;can be acivated with "0 0xD WREEP" and perform CPU reset
#if ((CPU==168) || (CPU==328))
	ldi AVIFLAGS,(m_HEX | m_HEXBACK | m_VREF0)	
							;assume hex number base as default
							;and ADC Vref=Vcc internally @ VREF pin
#else
	ldi AVIFLAGS,(m_HEX | m_HEXBACK | m_VREF0)	
							;assume hex number base as default
							;and ADC Vref=Vcc internally @ VREF pin
#endif
STD30:
	clr XFLAGS              ;among others, reset HISPEED flag
	clr YFLAGS

#if (CPU==3244)
	ldi r25,0x80
	out MCUCR,r25
	out MCUCR,r25			;disable JTAG as precaution
  #ifdef LEONARDO
	sbi DDRD,TXLED			;output
	cbi PORTD,TXLED      	;LED initially Port LOW=ON, 
  #else
	sbi DDRD,LED			;output
	sbi PORTD,LED      		;LED initially ON
  #endif

//;init Datastack pointer
	ldi XH,HIGH(DATASTACK)
	ldi XL,LOW(DATASTACK)	
		
#else  ; != 3244
  #if ((CPU==168) || (CPU==328))
	sbi DDRD,LED			;output
	sbi PORTD,LED      		;LED initially ON
  #else
	sbi DDRB,LED			;output
	sbi PORTB,LED      		;LED initially ON
  #endif

//;init Datastack pointer
	ldi XH,HIGH(DATASTACK)
	ldi XL,LOW(DATASTACK)
		
	rcall initUSART0
#endif  ;CPU!=3244

	in r25,ACSR				;disable Analog Comparator
	sbr r25,0x80           	;set Bit 7 = ACDisable
	out ACSR,r25

	call ABRT         		;specifically init variables and comntrolflags


;init Timer0: permanent overflow in 1 millisecond sequence
	rcall setTimer0

#if ((CPU==168) || (CPU==328))
;init interrup 0 reacting on negative edge (CNT)
	ldi r25,(1<<INT0)				;request enable
	out EIMSK,r25
	ldi r25,(1<<ISC01)				;sensing control
	sts EICRA,r25
#endif
#if ((CPU==644) || (CPU==1284))
;init interrups 2 and 0 reacting on negative edge (CNT, CNTX)
	ldi r25,(1<<INT0)|(1<<INT2)		;request enable
	out EIMSK,r25
	ldi r25,(1<<ISC01)|(1<<ISC21)	;sensing control
	sts EICRA,r25
#endif
#if (CPU==3244)
;init interrups 0 and 1 reacting on negative edge (CNT, CNTX)
	ldi r25,(1<<INT0)|(1<<INT1)		;request enable
	out EIMSK,r25
	ldi r25,(1<<ISC01)|(1<<ISC11)	;sensing control
	sts EICRA,r25
#endif
#if (CPU==32)
;init interrups 0 and 1 reacting on negative edge (CNT, CNTX)
	ldi r25,(1<<INT0)|(1<<INT1)		;request enable
	out GICR,r25
	ldi r25,(1<<ISC01)|(1<<ISC11)	;sensing control
	out MCUCR,r25
#endif

;make system sensitive
	sei
#if (CPU==3244)
	rcall initUSB    	    
	rcall delay50ms
STT40:
	wdr
	sbrs AVIFLAGS,USBCONNECTED
	rjmp STT40

	rcall delay50ms					;delay ca.250 milliseconds
	rcall delay50ms
	rcall delay50ms
	rcall delay50ms
	rcall delay50ms

;helpful for debugging when USB is critical
	call DUP
	clr TOSH
	ldi TOSL,BAUD384
	call USART1BAUD

#endif

#if (CPU==3244)
	jmp INITIOREADY ;performs AUTOEXE and jumps to QUIT
#else
	rjmp INITIOREADY ;performs AUTOEXE and jumps to QUIT
#endif

#if (CPU!=3244)
initUSART0:    ;*****************************************************
;********************************************************************
	ldi r25,LOW(CMDRXBUF)
	sts CMDRXHEADL,r25
	sts CMDRXTAILL,r25

	sbi DDRD,DDD1			;precaution, int TXD as output
	clr r25
  #if (CPU==32)
	out UBRRH,r25
  #else
	sts UBRR0H,r25
  #endif
   	#ifdef SPEED_1M8
	sbrs XFLAGS,HISPEED
	ldi r25,BAUD576 		;operates with 921kHZ compatible baud rate
	sbrc XFLAGS,HISPEED
	ldi r25,BAUDHI576		;operates with 1.843MHZ compatible baud rate
   	#endif
   	#ifdef SPEED_7M
	sbrs XFLAGS,HISPEED
	ldi r25,BAUD576 		;operates with 921kHZ compatible baud rate
	sbrc XFLAGS,HISPEED
	ldi r25,BAUDHI576		;operates with 7.37MHZ compatible baud rate
   	#endif
   	#ifdef SPEED_11M
	ldi r25,BAUD576
   	#endif
	#ifdef SPEED_14M
	ldi r25,BAUD576
	#endif
	#ifdef SPEED_16M
	  #ifdef ARDUINO_UNO
	   	ldi r25,1<<U2X0		;double Usart Baudrate
		sts UCSR0A,r25
		ldi r25,BAUD115 	;effectively divide 2MHz/17 =117.65kHz =2.1% error
	  #else
		ldi r25,BAUD384		;alternative 19200, others make inacceptable error
	  #endif
	#endif
   	#ifdef SPEED_18M
	ldi r25,BAUD576
   	#endif

  #if (CPU==32)
	out UBRRL,r25
;InitRX-Buffer in ABRT
	ldi r25,(1<<RXCIE)|(1<<RXEN)|(1<<TXEN) 	;0x98
	out UCSRB,r25
	ldi r25,(1<<URSEL)|(1<<USBS)|(1<<UCSZ1)|(1<<UCSZ0) ;0x1E
							;2 stopbits, 8bit data
							;URSEL is tricky to access UCSRS, see manual p158
	out UCSRC,r25
  #else   ;not ATmega32
  	sts UBRR0L,r25
;Init RX-Buffer in ABRT
	ldi r25,(1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0) 	;0x98
							;RXCIE,RXEN,TXEN - TX without interrupt
	sts UCSR0B,r25
	ldi r25,(1<<USBS0)|(1<<UCSZ01)|(1<<UCSZ00) ;0x0E
							;2 stopbits, 8bit data
	sts UCSR0C,r25
  #endif
  	ret

#endif  ;CPU!=3244

#if ((CPU==644) || (CPU==1284) || (CPU==3244))

USART1BAUD:   ;( b -- )  ********************************************
;set baud rate with original Atmel value(see Atmel manual) and init UART1
;baud rate = -1=0xFFFF closes USART1 and corresponding high level features
;********************************************************************
	mov r25,TOSL
	and r25,TOSH
	cpi r25,0xFF			;=-1
	brne U1B10				;disable
	cbr XFLAGS,m_USART1_ON
	jmp DROP

U1B10:
	ldi r25,LOW(US1RXBUF)
	sts US1RXHEAD,r25
	sts US1RXTAIL,r25

	sbi DDRD,3				;output
	sbi PORTD,3      		;initially High

	sts UBRR1L,TOSL
	ldi r25,(1<<RXCIE1)|(1<<RXEN1)|(1<<TXEN1) ;TX without interrupt
	sts UCSR1B,r25
	sbr XFLAGS,m_USART1_ON
	jmp DROP


RX1Q:   ;( -- 0 | b 1 )  ********************************************
;counterpart of KEY?
;********************************************************************
	sbrs XFLAGS,USART1_ON
	jmp runtimeErr

	st -X,TOSH				;DUP = release TOS
	st -X,TOSL
	sbrs YFLAGS,GETVABC		
	rjmp RXQ10				;due to CTRL_ABC, U1RX must be ignored
	clr TOSH				;else competition with pullCmdRxBuf
	clr TOSL
	ret
RXQ10:
	clr TOSH				;buffer is always one full 0x100 page
	lds TOSL,US1RXHEAD
	lds TEMP1,US1RXTAIL
	cp TOSL,TEMP1
	brcs RXQ20
	breq RXQ80
;HEAD is higher than TAIL
	sub TOSL,TEMP1
	ret
RXQ20:
;HEAD is lower than TAIL: add recv buffer length(=256 bytes) and subtract
	ldi TOSH,1
	clr TEMP2
	sub TOSL,TEMP1
	sbc TOSH,TEMP2
	ret
RXQ80:
	clr TOSL
	ret


U1RX:   ;( -- b )  **************************************************
;counterpart of KEY with explicit call of virtual "pullUSART1RxBuf"
;********************************************************************
	sbrs XFLAGS,USART1_ON
	jmp runtimeErr
U1R10:
	wdr
;check USART1 RX Buffer
	lds r25,US1RXHEAD
	lds r24,US1RXTAIL
	cp r24,r25
	breq U1R10
;load from RX Buffer
	lds ZL,US1RXTAIL
	ldi ZH,HIGH(US1RXBUF)	
	ld TOSL,Z+
	sts US1RXTAIL,ZL		;atomatic query tail wrap around(bufferSize=256)
	st -X,TOSH				;emulate DUP = release TOS
	st -X,TOSL			
	clr TOSH				;simply put 8bit CHAR on TOS
	ret


U1TX:   ;( b -- )  **************************************************
;counterpart of EMIT
;********************************************************************
	sbrs XFLAGS,USART1_ON
	jmp runtimeErr

	lds r25,UCSR1A
	sbrs r25,UDRE1
	rjmp U1TX
	sts UDR1,TOSL
	ld TOSL,X+				;emulate DROP
	ld TOSH,X+
	ret

#endif ;CPU=1284 etc

setTimer0:    ;******************************************************
;********************************************************************
;init Timer0: permanent overflow in 1 millisecond sequence
  #ifdef SPEED_18M
	ldi r25,4               ;prescale/256 gives 72.0 kHz @ 18,432M Crystal
  #endif   
   #ifdef SPEED_16M
  	ldi r25,4               ;prescale /256 gives 62.5 kHz @ 16.0M Crystal
  #endif       	
  #ifdef SPEED_14M
  	ldi r25,3               ;prescale /64 gives230.4 kHz @ 14.745M Crystal
  #endif
  #ifdef SPEED_11M
  	ldi r25,3               ;prescale /64 gives 172.80kHz @ 11.059M Crystal
  #endif
  #ifdef SPEED_7M			;starts with 921kHz CPU clock
   	ldi r25,2               ;prescale /8 gives 115.2 kHz @ init921.6kHz clock
	sbrc XFLAGS,HISPEED
  	ldi r25,3               ;prescale /64 gives gives 115.2 kHz @7.37 MHz clock
  #endif
  #ifdef SPEED_1M8			;starts with 921kHz CPU clock
  	ldi r25,2               ;prescale /8 gives 115.2 kHz @ 921.6kHz clock
							;or 230.4 kHz @ 1.84MHz clock
  #endif  												

#if (CPU==32)
	out TCCR0,r25
	ldi r25,(1<<TOIE0)
	out TIMSK,r25           		;Timer0 Overflow Interrupt enable
#else
	out TCCR0B,r25
	ldi r25,(1<<TOIE0)
	sts TIMSK0,r25       			;Timer0 Overflow Interrupt enable
#endif
	ret


#if (CPU==3244)
delay50ms:   ;*******************************************************
;delay ca.50 milliseconds
;********************************************************************
	wdr
	push r25
	ldi r25,250				
D5M10:
	wdr
	call delay200us
	dec r25
	brne D5M10
	pop r25
	ret
#endif

;--------------------------------------------------------------------
;Interrupt Routinen

#if (CPU != 3244) //USB receive interrupt see file AV5U.SAM

cmdSRXint:    ;******************************************************
;********************************************************************
	in SAVE_SREG,SREG
  #if (CPU==32)
	in INTREG,UDR              	;read and clear Interrupt Flag
  #else
	lds INTREG,UDR0            	;read and clear Interrupt Flag
  #endif

	sbrc AVIFLAGS,COMPILE
	rjmp CRI70
#if ((CPU==644) || (CPU==1284) || (CPU==3244))
	sbrc XFLAGS,NOBACKGND		;if set, no low level background messages
	rjmp CRI70
#endif
	cpi INTREG,ESC      		;ESC=brutal termination of endless loops etc
	brne CRI10
	out SREG,SAVE_SREG
	sei
	jmp ABORT					;shall ret to MAIN level
CRI10:
	cpi INTREG,CTRL_S      		;immediately send .S stack report
	brne CRI20
	push TEMP1
	push TEMP2
	push TEMP3
	push TEMP4
	push r24
	push r25
	push ZL
	push ZH
	call DOTS
	call DOTRS
	pop ZH
	pop ZL
	pop r25
	pop r24
	pop TEMP4
	pop TEMP3
	pop TEMP2
	pop TEMP1
	rjmp CRI80
CRI20:
	sbrc YFLAGS,GETVABC
	rjmp CRI70
	cpi INTREG,CTRL_A      	
	brne CRI30
	sbr YFLAGS,m_GETVABC | m_SAVEVA
	cbr YFLAGS,m_SAVEVB
	rjmp CRI50
CRI30:
	cpi INTREG,CTRL_B      	
	brne CRI40
	sbr YFLAGS,m_GETVABC | m_SAVEVB
	cbr YFLAGS,m_SAVEVA
	rjmp CRI50
CRI40:
	cpi INTREG,CTRL_C
	brne CRI70
	sbr YFLAGS,m_GETVABC | m_SAVEVA | m_SAVEVB
CRI50:
	push r24					;push/pop missed, bugfix 07July23
	ldi r24,CRR
	rcall sendCmdByte
	ldi r24,'V'
	rcall sendCmdByte
	ldi r24,'A'
	sbrs YFLAGS,SAVEVB			;bugfix 10Sept23
	rjmp CRI60
	ldi r24,'B'
	sbrs YFLAGS,SAVEVA
	rjmp CRI60
	ldi r24,'C'
CRI60:
	rcall sendCmdByte
	ldi r24,'?'
	rcall sendCmdByte
	pop r24
	clr INTREG
	sts BAKTIBPTR,INTREG
	out SREG,SAVE_SREG
	reti

CRI70:
;transfer received byte into buffer
	push YL
	push YH
	ldi YH,HIGH(CMDRXBUF)
	lds YL,CMDRXHEADL
	st Y+,INTREG				;automatic cyclic due to 1 full page buffer
	sts CMDRXHEADL,YL
	pop YH
	pop YL  
CRI80:
	out SREG,SAVE_SREG
	reti

#endif ;CPU!=3244


#if ((CPU==644) || (CPU==1284) || (CPU==3244))
US1SRXint:    ;******************************************************
;********************************************************************
 
	in SAVE_SREG,SREG
	lds INTREG,UDR1            ;read and clear Interrupt Flag
	push YL
	push YH
;transfer received byte into buffer
	ldi YH,HIGH(US1RXBUF)
	lds YL,US1RXHEAD
	st Y+,INTREG
	cpi YL,LOW(US1RXBUFTOP)
	brne U1RI10
	ldi YL,LOW(US1RXBUF)
U1RI10:  
	sts US1RXHEAD,YL
	pop YH
	pop YL 
	out SREG,SAVE_SREG
	reti

#endif

timer0Int:   ;*******************************************************
;count down MSCNTH,MSCNTL ca 1000 times per second
;********************************************************************
	in SAVE_SREG,SREG
	push YL
	push YH

	ldi INTREG,MSADJ
/*
#ifdef SPEED_7M
	sbrc XFLAGS,HISPEED
	ldi INTREG,MSADJHI		;MSHIADJ=MSADJ
#endif
*/
#ifdef SPEED_16M
	sbrs XFLAGS,XTIME
	rjmp TIN05
	cbr XFLAGS,m_XTIME
	dec INTREG				;adds 1/2 to timer interrupt countup
	rjmp TIN10
TIN05:  
	sbr XFLAGS,m_XTIME
TIN10:
#endif
#ifdef SPEED_14M
	sbrs XFLAGS,XTIME
	rjmp TIN05
	cbr XFLAGS,m_XTIME
	dec INTREG				;adds 1/2 to timer interrupt countup
	rjmp TIN10
TIN05:  
	sbr XFLAGS,m_XTIME
TIN10:
#endif
#ifdef SPEED_1M8
	sbrs XFLAGS,HISPEED
	rjmp TIN10				;operates with 921kHz
	ldi INTREG,MSADJHI
	sbrs XFLAGS,XTIME
	rjmp TIN05
	cbr XFLAGS,m_XTIME
	dec INTREG				;adds 1/2 to timer interrupt countup
	rjmp TIN10
TIN05:  
	sbr XFLAGS,m_XTIME
TIN10:
#endif
	out TCNT0,INTREG		;counts up until overflow

;MS, MSTIME process:
	lds YL,MSCNTL
	lds YH,MSCNTH
	mov INTREG,YH
	or INTREG,YL            ;no Countdown, if counter=0
	breq TIN15
	sbiw YL,1
	sts MSCNTL,YL
	sts MSCNTH,YH
TIN15:
;slow count processes, every 100 ms
	dec MS100CNT
	brne TIN20
	ldi INTREG,100
	mov MS100CNT,INTREG
	lds YL,TIXCNTL
	lds YH,TIXCNTH
	mov INTREG,YH
	or INTREG,YL            ;no Countdown, if counter=0
	breq TIN18
	sbiw YL,1
	sts TIXCNTL,YL
	sts TIXCNTH,YH
TIN18:
	push TEMP1
	push r24
	push r25
	push ZL
	push ZH
	call subRandom			;make pseudo random sequence more diffuse
	pop ZH
	pop ZL
	pop r25
	pop r24
	pop TEMP1

	tst LEDCNT
	breq TIN20
	lds YL,LEDAUTO
	tst YL
	brne TIN20
	dec LEDCNT
	brne TIN20
#ifdef LEONARDO
	cbi PORTD,TXLED         ;switch LED on again
#else
  #if ((CPU==644) || (CPU==1284) || (CPU==32))
	sbi PORTB,LED           ;switch LED on again
  #else
	sbi PORTD,LED           ;switch LED on again
  #endif
#endif
TIN20:  

#if (CPU==3244)
;handle CTRL_S background process  ----------------------------------
	sbrs YFLAGS,BAKDOTS
	rjmp TIN25
	nop
	push TEMP1
	push TEMP2
	push TEMP3
	push TEMP4
	push r24
	push r25
	push ZL
	push ZH
	call DOTS
	call DOTRS
	pop ZH
	pop ZL
	pop r25
	pop r24
	pop TEMP4
	pop TEMP3
	pop TEMP2
	pop TEMP1
	cbr YFLAGS,m_BAKDOTS
	rjmp TIN80
TIN25:
;handle VA,VB,VC background process  --------------------------------

	sbrs YFLAGS,GETVABC
	rjmp TIN80

	push TEMP1
	push INTREG
	push r24

	sbrc YFLAGS,NUMVABC
	rjmp TIN30
		
	sbr YFLAGS,m_NUMVABC

	ldi r24,CRR
	call sendCmdByte
	ldi r24,'V'
	call sendCmdByte

	sbrs YFLAGS,SAVEVA
	rjmp TIN27
	nop
	sbrc YFLAGS,SAVEVB
	rjmp TIN28
;only VA
	ldi r24,'A'
	call sendCmdByte
	rjmp TIN29
TIN27:
	sbrs YFLAGS,SAVEVB
	rjmp TIN72
;only VB
	ldi r24,'B'
	call sendCmdByte
	rjmp TIN29
TIN28:
;both SAVEVA and SAVEVB = VC
	ldi r24,'C'
	call sendCmdByte
TIN29:
	ldi r24,'?'
	call sendCmdByte
	clr INTREG
	sts BAKTIBPTR,INTREG
	rjmp TIN75

TIN30:
	wdr
 	call pullCmdRxBuf
	brts TIN32
	rjmp TIN75
TIN32:
	cpi TEMP1,4
	brcc TIN33
	rjmp TIN75
TIN33:
	mov r24,TEMP1
	cpi r24,CRR
	breq TIN40
	cpi r24,BKSP			;07Jul23
	breq TIN38
	cpi r24,0x7F
	brcs TIN34
	rjmp TIN75
TIN34:
	cpi r24,SPACE+1
	brcc TIN35
	rjmp TIN75				;ignore SPACE
TIN35:
	call sendCmdByte
	rjmp TIN40
TIN38:
	lds INTREG,BAKTIBPTR
	tst INTREG
	breq TIN30              ;ignore BS at begin of line
	dec INTREG				;clear character in TIB
	sts BAKTIBPTR,INTREG
	call sendCmdByte
	ldi r24,SPACE
	call sendCmdByte
	ldi r24,BKSP
	call sendCmdByte
	rjmp TIN30
TIN40:
	cpi r24,0x61
	brcs TIN45
	andi r24,0xDF			;clear bit5 = turn lower case into upper case			;
TIN45:
	lds INTREG,BAKTIBPTR
	ldi YH,HIGH(BAKTIB)
	ldi YL,LOW(BAKTIB)
	add YL,INTREG
	inc INTREG
	sts BAKTIBPTR,INTREG
	st Y,r24
	cpi r24,CRR
	breq TIN48
	rjmp TIN75
TIN48:
	ldi INTREG,SPACE
	st Y,INTREG
	
	push TEMP2
	push TEMP3
	push TEMP4
	push r25
	push ZL
	push ZH

	ldi YH,HIGH(BAKTIB)
	ldi YL,LOW(BAKTIB)
	lds STRINGLEN,BAKTIBPTR
	dec STRINGLEN			;CRR doesn't count
#if (CPU==3244)
	call checkNum			;returns 16Bit binary word in ZH(High),ZL(Low)
#else
	rcall checkNum			;returns 16Bit binary word in ZH(High),ZL(Low)
#endif
	brtc TIN65
	
	sbrs YFLAGS,SAVEVA
	rjmp TIN50
	sbrc YFLAGS,SAVEVB
	rjmp TIN55
;only VA
	sts VA_VALUE,ZL
	sts VA_VALUE+1,ZH
	rjmp TIN60
TIN50:
	sbrs YFLAGS,SAVEVB
	rjmp TIN65				;error then
;only VB
	sts VB_VALUE,ZL
	sts VB_VALUE+1,ZH
	rjmp TIN60
TIN55:
;both SAVEVA and SAVEVB = VC
	sts VC_VALUE,ZL
	sts VC_VALUE+1,ZH
TIN60:
	ldi r24,SPACE
	call sendCmdByte
	ldi r24,'o'
	call sendCmdByte
	ldi r24,'k'
	call sendCmdByte
	ldi r24,CRR
	call sendCmdByte
	rjmp TIN70
TIN65:
	ldi YH,HIGH(BAKTIB)
	ldi YL,LOW(BAKTIB)
	ldi r24,SPACE			;07JUL23
	st Y,r24
	ldi r24,'?'
	call sendCmdByte
	call sendCmdByte
	ldi r24,CRR	
	call sendCmdByte
TIN70:
	pop ZH
	pop ZL
	pop r25
	pop TEMP4
	pop TEMP3
	pop TEMP2

TIN72:
	cbr YFLAGS,m_NUMVABC | m_GETVABC | m_SAVEVA | m_SAVEVB

TIN75:
	pop r24
	pop INTREG
	pop TEMP1

;end of VA,VB,VC background process  --------------------------------

TIN80:  
	pop YH
	pop YL
	out SREG,SAVE_SREG
	reti

#else	;CPU not 3244  ------------------------------

;handle VA,VB,VC background process  --------------------------------
	sbrs YFLAGS,GETVABC
	rjmp TIN80

	push TEMP1
	push INTREG
	push r24
TIN30:
	wdr
 	rcall pullCmdRxBuf
	brts TIN32
	rjmp TIN75
TIN32:
	mov r24,TEMP1
	cpi r24,CRR
	breq TIN40
	cpi r24,BKSP			;07Jul23
	breq TIN38
	cpi r24,(SPACE + 1)
	brcc TIN35
	rjmp TIN75				;ignore SPACE and non-printables
TIN35:
	rcall sendCmdByte
	rjmp TIN40
TIN38:
	lds INTREG,BAKTIBPTR
	tst INTREG
	breq TIN30              ;ignore BS at begin of line
	dec INTREG				;clear character in TIB
	sts BAKTIBPTR,INTREG
	rcall sendCmdByte
	ldi r24,SPACE
	rcall sendCmdByte
	ldi r24,BKSP
	rcall sendCmdByte
	rjmp TIN30
TIN40:
	cpi r24,0x61
	brcs TIN45
	andi r24,0xDF			;clear bit5 = turn lower case into upper case			;
TIN45:
	lds INTREG,BAKTIBPTR
	ldi YH,HIGH(BAKTIB)
	ldi YL,LOW(BAKTIB)
	add YL,INTREG
	inc INTREG
	sts BAKTIBPTR,INTREG
	st Y,r24
	cpi r24,CRR
	breq TIN48
	rjmp TIN75
TIN48:
	ldi INTREG,SPACE
	st Y,INTREG

	push TEMP2
	push TEMP3
	push TEMP4
	push r25
	push ZL
	push ZH
	ldi YH,HIGH(BAKTIB)
	ldi YL,LOW(BAKTIB)
	lds STRINGLEN,BAKTIBPTR
	dec STRINGLEN				;CRR doesn't count

	sbr YFLAGS,m_NUMVABC		;suppresses double OK msg

	rcall checkNum
	brtc TIN65
	sbrs YFLAGS,SAVEVA
	rjmp TIN50
	sbrc YFLAGS,SAVEVB
	rjmp TIN55
;only VA
	sts VA_VALUE,ZL
	sts VA_VALUE+1,ZH
	rjmp TIN60
TIN50:
	sbrs YFLAGS,SAVEVB
	rjmp TIN65					;error then
;only VB
	sts VB_VALUE,ZL
	sts VB_VALUE+1,ZH
	rjmp TIN60
TIN55:
;VA and VB = VC
	sts VC_VALUE,ZL
	sts VC_VALUE+1,ZH
TIN60:
	ldi r24,SPACE
	rcall sendCmdByte
	ldi r24,'o'
	rcall sendCmdByte
	ldi r24,'k'
	rcall sendCmdByte
	ldi r24,CRR
	rcall sendCmdByte
	rjmp TIN70
TIN65:
	ldi YH,HIGH(BAKTIB)
	ldi YL,LOW(BAKTIB)
	ldi r24,SPACE			;07JUL23
	st Y,r24
	ldi r24,'?'
	rcall sendCmdByte
	rcall sendCmdByte
	ldi r24,CRR
	rcall sendCmdByte
TIN70:
	pop ZH
	pop ZL
	pop r25
	pop TEMP4
	pop TEMP3
	pop TEMP2

	cbr YFLAGS,m_NUMVABC | m_GETVABC | m_SAVEVA | m_SAVEVB

TIN75:
	pop r24
	pop INTREG
	pop TEMP1

;end of VA,VB,VC background process  --------------------------------

TIN80:  
	pop YH
	pop YL
	out SREG,SAVE_SREG
	reti
#endif

countInt0:   ;*******************************************************
; triggered by change at INT0 pin
; CNT0PIN pin processor type specific, all processor types PD2
; except ATmega32U4:PD0
; ATmega644,1284:conflict with USART1
;********************************************************************
	in SAVE_SREG,SREG
	push ZL
	push ZH
	lds ZH,CNT0H            ;separate count register
	lds ZL,CNT0L
	adiw ZL,1
	sts CNT0H,ZH
	sts CNT0L,ZL

	lds ZH,ENC0H
	lds ZL,ENC0L
	sbiw ZL,1
#if ((CPU==168) | (CPU==328))
	sbic PIND,ENC0PIN			;PD5
#endif
#if ((CPU==644) | (CPU==1284))
	sbic PINB,ENC0PIN			;PB1
#endif
#if (CPU==3244)
	sbic PIND,ENC0PIN			;PD4
#endif
#if (CPU==32)
	sbic PINB,ENC0PIN			;PB7
#endif
	adiw ZL,2
	sts ENC0H,ZH
	sts ENC0L,ZL
	pop ZH
	pop ZL
	out SREG,SAVE_SREG
	reti

#if ((CPU==32) || (CPU==3244) || (CPU==644) || (CPU==1284))
countIntX:   ;*******************************************************
; triggered by change at INT1 pin (ATmega644,1284:INT2)
; CNTXPIN pin processor type specific:mega32U4:PD1-mega32:PD3,else:PB2
;********************************************************************
	in SAVE_SREG,SREG
	push ZL
	push ZH
	lds ZH,CNTXH            ;separate count register
	lds ZL,CNTXL
	adiw ZL,1
	sts CNTXH,ZH
	sts CNTXL,ZL

	lds ZH,ENCXH
	lds ZL,ENCXL
	sbiw ZL,1
#if ((CPU==644) || (CPU==1284))
	sbic PINB,ENCXPIN		;PB3
#endif
#if (CPU==3244)
	sbic PIND,ENCXPIN		;PD5
#endif
#if (CPU==32)
	sbic PINB,ENCXPIN		;PB7
#endif
	adiw ZL,2
	sts ENCXH,ZH
	sts ENCXL,ZL
	pop ZH
	pop ZL
	out SREG,SAVE_SREG
	reti
#endif	

sleepInt:  ;*********************************************************
;release from SLEEP is triggered by pin change external interrupt
;ATmega168,328,644,1284: PD0(USART0 RX) or PD4
;ATmega32:PB2
;SLEEP not provided for ATmega32U4
;********************************************************************
	in SAVE_SREG,SREG
	nop
	nop
	out SREG,SAVE_SREG
	reti


#if (CPU==3244)
	.INCLUDE "AV5U.ASM"
#endif
;####################################################################

	.INCLUDE "AV5K.ASM"     ;essentially contains kernel code
	.INCLUDE "AV5S.ASM"     ;"S" for "special I/O-functions"and kernel names

;####################################################################

;Bootloader
;----------
	.ORG USERFLASH
/*
#if (CPU==168)
	.ORG 0x1F80
#endif
#if (CPU==32)
	.ORG 0x3F00
#endif
#if (CPU==328)
	.ORG 0x3F00
#endif
#if (CPU==3244)
	.ORG 0x3F00
#endif
#if (CPU==644)
	.ORG 0x7E00
#endif
#if (CPU==1284)
	.ORG 0xFE00
#endif
*/

progFlash:    ;******************************************************
;Parameter: XH,XL=SRAM-source address. Save datastack pointer before !
;           ZH,ZL=Flash-target address
;r0,r1 are destroyed/overwritten
;********************************************************************
;Page Erase
	push r1                 
	push TEMP1              	;will be overwritten by doSPM
	push r24
	push r25
	push ZL
	push ZH

	cpi ZH,HIGH(KERNELTOP*2)	;BYTES,ensures page limit alignment			
	brcc PRF10
	jmp memoryError         	;protect programm code

PRF10:
	cpi ZH,HIGH(USERFLASH*2)	;BYTES, must be on page limit!
	brcs PRF15
	jmp memoryError         	;protect programm code

PRF15:

;check if EEPROM is not writing
PRF20:  
	sbic EECR,EEPE
	rjmp PRF20

	cli

;check if UART in not transmitting
#if (CPU==32)
PRF30:  
	sbis UCSRA,TXC
	rjmp PRF30
#else
  #if (CPU != 3244)
PRF30:  
	lds r24,UCSR0A
	andi r24,1<<TXC0
	breq PRF30
  #endif
#endif
/*
#if ((CPU==644)||(CPU==1284))
PRF35:   
	lds r24,UCSR1A
	sbrs r24,TXC1
	rjmp PRF35
#endif
*/
	ldi r25,((1<<PGERS)|(1<<SPMEN))   ;3
	rcall doSPM
;Re-Enable PWW Section
	ldi r25,((1<<RWWSRE)|(1<<SPMEN))  ;0x11
	rcall doSPM

;transfer data from CBUF into flash burn buffer
	ldi r24,FLASHPAGE	;=PAGESIZE transfer two bytes per loop
PRF40:
	wdr
	ld r0,X+
	ld r1,X+
	ldi r25,(1<<SPMEN)
	rcall doSPM
	adiw ZL,2
	dec r24
	brne PRF40

;execute PageWrite
	pop ZH
	pop ZL
	push ZL
	push ZH
	ldi r25,((1<<PGWRT) | (1<<SPMEN))
	rcall doSPM
;Re-Enable PWW Section
	ldi r25,((1<<RWWSRE)|(1<<SPMEN))  ;0x11
	rcall doSPM

;check if RWW section free again, continue then
PRF60:
	in r24,SPMCSR
	sbrs r24,RWWSB
	rjmp PRF80
	ldi r25,((1<<RWWSRE)|(1<< SPMEN))
	rcall doSPM
	rjmp PRF60
PRF80:

	sei

	pop ZH
	pop ZL
	pop r25
	pop r24
	pop TEMP1
	pop r1
	ret


doSPM:    ;**********************************************************
; parameter r25: data for SPMCR
;********************************************************************
DSM20:
	in TEMP1,SPMCSR
	sbrc TEMP1,SPMEN
	rjmp DSM20

;execute SPM-4 clock sequence
	out SPMCSR,r25
	spm

	ret

	nop
	nop


;--------------------------------------------------------------------

CARRYY:   ;  ( -- 1 | 0 )  ******************************************
;********************************************************************
	st -X,TOSH
	st -X,TOSL				;emulate DUP
	lds TOSH,FCARRYH
	lds TOSL,FCARRYL
	ret


RANDOMM:   ;  ( -- random )  ****************************************
;return random number
;********************************************************************
	st -X,TOSH
	st -X,TOSL				;emulate DUP
	rcall subRandom
	lds TOSH,RANDOMH
	lds TOSL,RANDOML
	ret


subRandom:   ;  *****************************************************
;calculate xorshift16 from RANDOMH/RANDOML as seed and store result back there
;uses "<<7, >>9, <<8" algoritm
;********************************************************************
	lds ZH,RANDOMH
	lds ZL,RANDOML
;<<7 step
	mov r24,ZH
	ror r24
	andi r24,0x80
	mov r25,ZL
	ror r25
	mov TEMP1,r25
	andi r25,0x80
	andi TEMP1,0x7F
	or r24,TEMP1
	eor ZH,r24
	eor ZL,r25
;>>9 step
	mov r24,ZH
	lsr r24
;ZH is not modified because eor'ed with zero
	eor ZL,r24
;<<8 step
	eor ZH,ZL
;ZL is not modified because eor'ed with zero
	sts RANDOMH,ZH
	sts RANDOML,ZL
	ret


QUIET:  ;  ( T|F -- )  **********************************************
;TRUE deactivates response of terminal input
;by default, terminal input is resonded.
;********************************************************************
	cbr AVIFLAGS,m_TQUIET
	or TOSL,TOSH
	tst TOSL
	breq QIE10
	sbr AVIFLAGS,m_TQUIET
QIE10:
	jmp DROP			


LEDMODE:  ; ( 0|1|2 --  )  ********************************************
;LED auto/manual on/off
;0=terminal blink, 1=OFF, 2=ON
;*********************************************************************
	cpi TOSL,3
	brcc LMD10

	sts  LEDAUTO,TOSL	;0=AUTO, 1=OFF, 2=ON
	cpi TOSL,1
	brne LMD10
#ifdef LEONARDO
	sbi PORTD,TXLED         ;switch LED OFF
#else
  #if ((CPU==644) || (CPU==1284) || (CPU==32))
	cbi PORTB,LED           ;switch LED OFF
  #else
	cbi PORTD,LED           ;switch LED OFF
  #endif
#endif
	jmp DROP
LMD10:
#ifdef LEONARDO
	cbi PORTD,TXLED         ;switch LED ON
#else
  #if ((CPU==644) || (CPU==1284) || (CPU==32))
	sbi PORTB,LED           ;switch LED ON
  #else
	sbi PORTD,LED           ;switch LED ON
  #endif
#endif
	jmp DROP

LMD90:
	jmp parameterErr


#if (CPU != 168)

#ifdef SPEED_7M
SPEED:  ; ( 1|2 --  )  ***********************************************
;change system speed during runtime
;before use: insert a 1ms pause to finish any USART transmission(if active)
;*********************************************************************
	cpi TOSL,2
	brcc SPD10
;switch down to 921kHz clock
	ldi r24,0x80			
	ldi r25,3				;2 step clock prescaler init procedure
	sts CLKPR,r24			;only write CLKPCE bit
	sts CLKPR,r25			;set division/8
	cbr XFLAGS,m_HISPEED
	rjmp SPD40
SPD10:
;switch up to 7.37MHz clock;
	ldi r24,0x80			
	clr r25					;2 step clock prescaler init procedure
	sts CLKPR,r24			;only write CLKPCE bit
	sts CLKPR,r25			;set division/1 (CPU clock = crystal freq)
	sbr XFLAGS,m_HISPEED
SPD40:
	call setTimer0			;permanent overflow in 1 millisecond sequence
	call initUSART0

#if ((CPU==644) || (CPU==1284))
	sbrs XFLAGS,USART1_ON	
	jmp DROP
	lds r24,UBRR1L
	inc r24
	tst TOSL
	brne SPD50				
;->LOW speed
	lsr r24
	lsr r24
	lsr r24			// /8
	rjmp SPD60
SPD50:						;->HIGH speed
	lsl r24
	lsl r24
	lsl r24			//*8
SPD60:
	dec r24
	sts UBRR1L,r24
#endif
	jmp DROP

#endif

#ifdef SPEED_1M8
SPEED:  ; ( 1|2 --  )  ***********************************************
;change system speed during runtime
;before use: insert a 1ms pause to finish any USART transmission(if active)
;*********************************************************************
	cpi TOSL,2
	brcc SPD10
;switch down to 921kHz clock
	ldi r24,0x80			
	ldi r25,1				;2 step clock prescaler init procedure
	sts CLKPR,r24			;only write CLKPCE bit
	sts CLKPR,r25			;set division/2
	cbr XFLAGS,m_HISPEED
	rjmp SPD40
SPD10:
;switch up to 1.8432MHz clock;
	ldi r24,0x80			
	clr r25					;2 step clock prescaler init procedure
	sts CLKPR,r24			;only write CLKPCE bit
	sts CLKPR,r25			;set division/1 (CPU clock = crystal freq)
	sbr XFLAGS,m_HISPEED
SPD40:
	call initUSART0
#if ((CPU==644) || (CPU==1284))
	sbrs XFLAGS,USART1_ON	
	jmp DROP
	lds r24,UBRR1L
	inc r24
	tst TOSL
	brne SPD50				
;->LOW speed
	lsr r24			// /2
	rjmp SPD60
SPD50:						;->HIGH speed
	lsl r24			//*2
SPD60:
	dec r24
	sts UBRR1L,r24
#endif
	jmp DROP

#endif

CODEOF:   ;**********************************************************
;********************************************************************
	clt
	call scanNext          ;get Z=pointer to next text word in query

	tst STRINGLEN           ;no argument
	breq CVC90

	mov YL,ZL               ;works as TIB pointer in "findUserToken"
	call findToken     	;returns token value High Byte in TEMP5
							;and token value Low Byte in TEMP2
							;or jumps directly to QUIT if no valid token
	brtc CVC90				;UserToken not found

	sbrc AVIFLAGS,COMPILE
	rjmp CVC10
	st -X,TOSH
	st -X,TOSL				;emulate DUP
	mov TOSL,TEMP2
	mov TOSH,TEMP5		
	ret

CVC10:
;Compile:put immediate Value-Adress on Stack
	ldi ZH,HIGH(JDOLIT)		;compile target addr
	ldi ZL,LOW(JDOLIT)
	call compiXToken       ;compile equivalent of DOLIT
	ret

CVC90:
;no matching string found  26Aug23
	jmp nameError


VCALL:   ;***********************************************************
;********************************************************************
	lds ZL,VC_VALUE
	lds ZH,VC_VALUE+1		;VC_VALUE uses little endian
	icall    
	ret

#endif ;(CPU != 168)



.NOLIST

;####################################################################
; Revision History

;05.Feb23  AVISE 5.5
;			copied from Avise44. Supported CPU Types and Speed Options renewed
;			ATmega128,162,163,8515,8535,8,88 deleted
;			new: ATmega328,644,1284. ATmega32 only old remained type
;			Speeds only 11M,14M,16M,18M, new Low Power 921K und 1843k
;			first prototypes with ATmega32,168,328
;10Feb23	essential bugfix:correct transfer code pointer from Flash to SRAM
;13Feb23	processToken Parameter r0 instead of TEMP3
;			DEBUG and BREAKK deleted, DEBUGGING obsolete
;			-->AVISE 5.6
;			Ports init by default as input with pull-up
;03March23	first running version with ATmega328P on Arduino Uno
;05March23	first running versions with ATmega644,1284
;06March23	first running version with ATmega32U4 USB-COM (Arduino Micro)
;10May23	ATmega32U4 COM transmission interrupt driven->much higher speed
;16May23	Kernel based Variables VA,VB,VC implemented
;18May23	background operations implemented: CTRL-A,B,C, CTRL-S, CTRL-T, ESC
;June 23    a lot if fine tuning, some minor bugfix
;30 June23  VISE5.6 published
;			-->AVISE5.7
;03July23	IFGO renamed doWHILE
;			GO doubled ->quasi dummy DOAGAIN(different name,same runtime code)
;			IFNOTGOGO doubled ->quasi dummy doUNTIL
;			GO renamed doELSE
;			IFNOTGO renamed doIF
;05July23	subroutine DOTS stack depth test improved
;07July23	bugfix cmdSRXint (push/pop r24 at CTRL_A,B,C)
;			terminal response CTRL_A,B,C improved
;			terminal output of MEM improved
;			AUTOEXE start improved
;09July23	Bugfix User Symbol Table: crash if code address low byte=0xFF
;			User Symbol Table reorg: end marked by double 0xFF 0xFF
;			reAddressVars adapted
;10July23	RAMDOM new, some internal parameters reorganized
;11July23   bugfix (KEY? and MS concerning CTRL_A,B,C) 
;13July23   op KEY: suppression of CTRL_K transfer on stack removed. Due to
;			missing stack entry, subsequent code may get stablility problems
;13July23	Avise 5.7 published
;			-->AVISE 5.8
;15July23   RANDOM, CARRY, QT, LEDMODE put into boot flash. 
;			--> 1 page more user code for ATmega168, 328 and 32
;24Aug23	SLEEP: USART not re-init, LED on/off improved
;26Aug23	AUTOEXE improved: User Functions only
;27Aug23    CODEOF and VCALL added
;06Sept23  	token2Str called by SEE: bugfix kernel code in BootLoader Section 
;			bugfix:setTimer0 extra subroutine, inserted in SPEED code
;			bugfix SPEED_7M: MSADJHI
;09Sept23	bugfix QUIT error messages
;			MEM improved: prints user specified page
;10Sept23	bugfix cmdSRXint (VC detection)
;14Sept23	output of User Function address count byte added in SEE
;			AREF switches ADC off (saves some current if not needed anymore)
;			Kernel Operators rearranged in Symbol Table (listed by OPS)
;			ABSCHEK renamed DIFCHEK, BINCHEK renamed NEWCHECK
;15Sept23	improvement 0 MEM output for ATmega1284 (unsigned decimal number)
;17Sept23	'unsigned' improvement from 15Sept generalized to Kernel Op .U	
;			Kernel Symbol Table moved to top of Kernel code
;			-->more "rcall" instructions  in user code
;18Sept23	Avise5.8 published
;			--> AVIDSE 5.9
;02Oct23	printout of SEE optimized (parameter in parenthesis) and compacted
;25Oct23	MSTIME renamed TIX, MS50CNT renamed MS100CNT (now 1/10s steps)
;			separate counters for MS and TIX(ie up to 6553 sec now)
;			LEDCNT values and RANDOM update changed correspondingly
;29Oct23	kernel ops DIFCHEK and NEWCHEk removed
;31Oct23    bugfix compare16()  (wrong cumpare results)	
;			comparison operators (signed compare) improved
;			FLUSH renamed to FS
;02Nov23	CONTI new,BREAK and CONTI useable for DO..AGAIN,UNTIL,WHILE loops
;			for this reason operators RS-1UP and RS-4UP introduced
;			useful for correct RETURN out of loops too.
;03Nov23	return stack test .RS implemented	
;			Return Stack organisation in loops improved
;04Nov23	printout of signed hex numbers improved (Kernel Ops . .H .D .U)
;05Nov23    kernel op PICK implemented
;07Nov23	RX1? and RX1 improved. PX1? returns number of received bytes, 
;			RS1 works independently of RX1?  KEY improved same way
;			USART1 baudrate accordingly updated with SPEED
;			bugfix ATmega32U4 VID PID Kernel Ops
;08Nov23	option DEFNUMB in EEProm new: change system start numberBase with
;			"0 0xD WREEP"-->HEX  or  "0xFF 0xD WREEP"-->DECIMAL			
;10Nov23	SPEED user parameters changed from 0/1 to 1/2,now<=1:LOW,>=2:HIGH
;			only versions 1m8 and 7m, others unmodified from 08Nov

;####################################################################
	.EXIT

